# !-*- coding:utf-8 -*-

# This document was created from Midas2OPS version 0.01
import json
from collections import namedtuple
import opensees.openseespy


def Igloo():
    # Set the default model dimensions and number of dofs.
    ops = opensees.openseespy.Model('basic', '-ndm', 3, '-ndf', 3)

    # Mechanical properties of material
    # E-Elastic Modulus; 
    # G-Shear Modulus; 
    # v-Poisson's ratio; 
    # rho-Mass Density;
    # gamma-unit weight; 
    # r: expansion coefficient
    mat_props = namedtuple(
        'mat_props', ['mat_name', 'E', 'v', 'G', 'r', 'gamma', 'rho'])
    MatProps = dict()
    MatProps[1] = mat_props._make(
        ('C280', 2824300.0, 0.18, 1196737.2881355933, 1e-05, 2.5, 0.0))

    # Create OpenSees nodes

    # NODES
    with open("nodes.json", "r") as f:
        for node in json.load(f):
            ops.node(*node)

    # Solid element.
#   ops.eval('model basic -ndm  3 -ndf  3')

    ops.nDMaterial('ElasticIsotropic', 1, MatProps[1].E, MatProps[1].v)

    # ELEMENTS
    with open("elements.json", "r") as f:
        for elem in json.load(f):
            typ      = elem[0] # name of element type
            tag      = elem[1] # unique identifier for element
            nodes    = elem[2] # list containing node tags
            material = elem[3]

            ops.element(typ, tag, *nodes, material)


    # Fix the node.
    ops.fix( 167, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 168, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 169, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 170, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 171, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 172, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 212, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 221, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 240, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 245, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 270, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 275, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 280, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 285, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 305, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 310, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 315, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 336, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 337, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 338, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 339, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 379, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 380, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 381, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 382, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 383, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 384, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 385, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 411, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 481, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 494, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 514, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 515, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 516, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 702, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 703, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 704, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 705, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 706, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 707, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 708, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 709, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 710, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 711, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 712, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 713, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 714, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 715, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 716, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 717, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 718, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 719, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 720, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 721, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 722, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 723, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 724, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 725, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 726, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 727, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 728, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 729, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 730, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 731, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 732, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 733, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 734, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 735, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 736, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 737, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 738, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 739, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 740, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 741, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 742, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 743, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 744, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 745, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 746, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 747, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 748, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 749, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 750, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 751, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 752, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 753, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 754, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 755, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 756, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 757, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 758, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 759, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 760, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 761, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 762, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 763, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 764, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 765, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 766, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 767, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 768, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 769, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 770, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 771, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix( 772, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1761, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1773, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1774, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1775, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1812, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1813, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1815, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1894, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1928, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1939, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1940, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1983, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1984, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(1985, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2005, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2042, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2087, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2089, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2090, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2136, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2137, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2141, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2161, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2162, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2163, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2165, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2166, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2177, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2180, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2181, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2182, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2183, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2265, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2295, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2304, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2305, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2330, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2353, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2355, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2372, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2374, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2394, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2397, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2399, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2400, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2401, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2437, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2439, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2461, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2472, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2473, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2477, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2521, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2546, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2557, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2571, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2582, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2585, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2586, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2599, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2607, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2613, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2623, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2628, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2629, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2642, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2644, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2648, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2649, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2650, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2651, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2654, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2663, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2666, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2671, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2672, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2674, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2678, *[1, 1, 1]) #, 1, 1, 1])
    ops.fix(2683, *[1, 1, 1]) #, 1, 1, 1])

    # Set the mass at a node.
    ops.mass(135, *[0.0003, 0.0003, 0.0003])
    ops.mass(136, *[0.0002, 0.0002, 0.0002])
    ops.mass(137, *[0.0004, 0.0004, 0.0004])
    ops.mass(138, *[0.0006, 0.0006, 0.0006])
    ops.mass(139, *[0.0004, 0.0004, 0.0004])
    ops.mass(140, *[0.0005, 0.0005, 0.0005])
    ops.mass(141, *[0.0006, 0.0006, 0.0006])
    ops.mass(142, *[0.0004, 0.0004, 0.0004])
    ops.mass(143, *[0.0003, 0.0003, 0.0003])
    ops.mass(144, *[0.0002, 0.0002, 0.0002])
    ops.mass(145, *[0.0002, 0.0002, 0.0002])
    ops.mass(146, *[0.0009, 0.0009, 0.0009])
    ops.mass(147, *[0.0006, 0.0006, 0.0006])
    ops.mass(148, *[0.0006, 0.0006, 0.0006])
    ops.mass(149, *[0.0004, 0.0004, 0.0004])
    ops.mass(150, *[0.0011, 0.0011, 0.0011])
    ops.mass(151, *[0.0005, 0.0005, 0.0005])
    ops.mass(152, *[0.001, 0.001, 0.001])
    ops.mass(153, *[0.0006, 0.0006, 0.0006])
    ops.mass(154, *[0.0007, 0.0007, 0.0007])
    ops.mass(155, *[0.0019, 0.0019, 0.0019])
    ops.mass(156, *[0.0014, 0.0014, 0.0014])
    ops.mass(157, *[0.0011, 0.0011, 0.0011])
    ops.mass(158, *[0.0014, 0.0014, 0.0014])
    ops.mass(159, *[0.002, 0.002, 0.002])
    ops.mass(160, *[0.0016, 0.0016, 0.0016])
    ops.mass(161, *[0.0018, 0.0018, 0.0018])
    ops.mass(162, *[0.0015, 0.0015, 0.0015])
    ops.mass(163, *[0.003, 0.003, 0.003])
    ops.mass(164, *[0.0017, 0.0017, 0.0017])
    ops.mass(165, *[0.0011, 0.0011, 0.0011])
    ops.mass(166, *[0.0004, 0.0004, 0.0004])
    ops.mass(167, *[0.0004, 0.0004, 0.0004])
    ops.mass(168, *[0.0008, 0.0008, 0.0008])
    ops.mass(169, *[0.0007, 0.0007, 0.0007])
    ops.mass(170, *[0.0004, 0.0004, 0.0004])
    ops.mass(171, *[0.0005, 0.0005, 0.0005])
    ops.mass(172, *[0.0004, 0.0004, 0.0004])
    ops.mass(173, *[0.0008, 0.0008, 0.0008])
    ops.mass(174, *[0.0004, 0.0004, 0.0004])
    ops.mass(175, *[0.0019, 0.0019, 0.0019])
    ops.mass(176, *[0.0014, 0.0014, 0.0014])
    ops.mass(177, *[0.0024, 0.0024, 0.0024])
    ops.mass(178, *[0.001, 0.001, 0.001])
    ops.mass(179, *[0.0001, 0.0001, 0.0001])
    ops.mass(180, *[0.0002, 0.0002, 0.0002])
    ops.mass(181, *[0.0001, 0.0001, 0.0001])
    ops.mass(182, *[0.0002, 0.0002, 0.0002])
    ops.mass(183, *[0.0003, 0.0003, 0.0003])
    ops.mass(184, *[0.0003, 0.0003, 0.0003])
    ops.mass(185, *[0.0005, 0.0005, 0.0005])
    ops.mass(186, *[0.0004, 0.0004, 0.0004])
    ops.mass(187, *[0.0003, 0.0003, 0.0003])
    ops.mass(188, *[0.0009, 0.0009, 0.0009])
    ops.mass(189, *[0.0006, 0.0006, 0.0006])
    ops.mass(190, *[0.0004, 0.0004, 0.0004])
    ops.mass(212, *[0.0003, 0.0003, 0.0003])
    ops.mass(213, *[0.001, 0.001, 0.001])
    ops.mass(214, *[0.0008, 0.0008, 0.0008])
    ops.mass(215, *[0.0006, 0.0006, 0.0006])
    ops.mass(216, *[0.0006, 0.0006, 0.0006])
    ops.mass(217, *[0.001, 0.001, 0.001])
    ops.mass(218, *[0.0007, 0.0007, 0.0007])
    ops.mass(219, *[0.0006, 0.0006, 0.0006])
    ops.mass(220, *[0.0009, 0.0009, 0.0009])
    ops.mass(221, *[0.0003, 0.0003, 0.0003])
    ops.mass(222, *[0.0006, 0.0006, 0.0006])
    ops.mass(223, *[0.0007, 0.0007, 0.0007])
    ops.mass(224, *[0.0006, 0.0006, 0.0006])
    ops.mass(225, *[0.0005, 0.0005, 0.0005])
    ops.mass(229, *[0.0008, 0.0008, 0.0008])
    ops.mass(230, *[0.001, 0.001, 0.001])
    ops.mass(231, *[0.0005, 0.0005, 0.0005])
    ops.mass(232, *[0.0011, 0.0011, 0.0011])
    ops.mass(233, *[0.0007, 0.0007, 0.0007])
    ops.mass(234, *[0.0005, 0.0005, 0.0005])
    ops.mass(235, *[0.0008, 0.0008, 0.0008])
    ops.mass(236, *[0.0006, 0.0006, 0.0006])
    ops.mass(237, *[0.0005, 0.0005, 0.0005])
    ops.mass(238, *[0.0006, 0.0006, 0.0006])
    ops.mass(239, *[0.0004, 0.0004, 0.0004])
    ops.mass(240, *[0.0003, 0.0003, 0.0003])
    ops.mass(241, *[0.0004, 0.0004, 0.0004])
    ops.mass(242, *[0.0006, 0.0006, 0.0006])
    ops.mass(243, *[0.0004, 0.0004, 0.0004])
    ops.mass(244, *[0.0005, 0.0005, 0.0005])
    ops.mass(245, *[0.0002, 0.0002, 0.0002])
    ops.mass(246, *[0.0008, 0.0008, 0.0008])
    ops.mass(247, *[0.0007, 0.0007, 0.0007])
    ops.mass(248, *[0.0005, 0.0005, 0.0005])
    ops.mass(249, *[0.0005, 0.0005, 0.0005])
    ops.mass(250, *[0.0003, 0.0003, 0.0003])
    ops.mass(251, *[0.001, 0.001, 0.001])
    ops.mass(252, *[0.0004, 0.0004, 0.0004])
    ops.mass(253, *[0.0005, 0.0005, 0.0005])
    ops.mass(254, *[0.0006, 0.0006, 0.0006])
    ops.mass(255, *[0.001, 0.001, 0.001])
    ops.mass(256, *[0.0006, 0.0006, 0.0006])
    ops.mass(257, *[0.0003, 0.0003, 0.0003])
    ops.mass(258, *[0.0008, 0.0008, 0.0008])
    ops.mass(259, *[0.0012, 0.0012, 0.0012])
    ops.mass(260, *[0.0005, 0.0005, 0.0005])
    ops.mass(261, *[0.0008, 0.0008, 0.0008])
    ops.mass(262, *[0.0006, 0.0006, 0.0006])
    ops.mass(263, *[0.0007, 0.0007, 0.0007])
    ops.mass(264, *[0.0004, 0.0004, 0.0004])
    ops.mass(265, *[0.0007, 0.0007, 0.0007])
    ops.mass(266, *[0.0006, 0.0006, 0.0006])
    ops.mass(267, *[0.0007, 0.0007, 0.0007])
    ops.mass(268, *[0.0002, 0.0002, 0.0002])
    ops.mass(269, *[0.0004, 0.0004, 0.0004])
    ops.mass(270, *[0.0001, 0.0001, 0.0001])
    ops.mass(271, *[0.0009, 0.0009, 0.0009])
    ops.mass(272, *[0.0011, 0.0011, 0.0011])
    ops.mass(273, *[0.0012, 0.0012, 0.0012])
    ops.mass(274, *[0.0012, 0.0012, 0.0012])
    ops.mass(275, *[0.0004, 0.0004, 0.0004])
    ops.mass(276, *[0.0012, 0.0012, 0.0012])
    ops.mass(277, *[0.0008, 0.0008, 0.0008])
    ops.mass(278, *[0.0011, 0.0011, 0.0011])
    ops.mass(279, *[0.0012, 0.0012, 0.0012])
    ops.mass(280, *[0.0007, 0.0007, 0.0007])
    ops.mass(281, *[0.0008, 0.0008, 0.0008])
    ops.mass(282, *[0.0014, 0.0014, 0.0014])
    ops.mass(283, *[0.0009, 0.0009, 0.0009])
    ops.mass(284, *[0.0012, 0.0012, 0.0012])
    ops.mass(285, *[0.0005, 0.0005, 0.0005])
    ops.mass(286, *[0.0006, 0.0006, 0.0006])
    ops.mass(287, *[0.001, 0.001, 0.001])
    ops.mass(288, *[0.0014, 0.0014, 0.0014])
    ops.mass(289, *[0.001, 0.001, 0.001])
    ops.mass(290, *[0.0008, 0.0008, 0.0008])
    ops.mass(291, *[0.0004, 0.0004, 0.0004])
    ops.mass(292, *[0.0007, 0.0007, 0.0007])
    ops.mass(293, *[0.0008, 0.0008, 0.0008])
    ops.mass(294, *[0.0009, 0.0009, 0.0009])
    ops.mass(295, *[0.001, 0.001, 0.001])
    ops.mass(296, *[0.0007, 0.0007, 0.0007])
    ops.mass(297, *[0.0008, 0.0008, 0.0008])
    ops.mass(298, *[0.0006, 0.0006, 0.0006])
    ops.mass(299, *[0.0007, 0.0007, 0.0007])
    ops.mass(300, *[0.0006, 0.0006, 0.0006])
    ops.mass(301, *[0.0013, 0.0013, 0.0013])
    ops.mass(302, *[0.0003, 0.0003, 0.0003])
    ops.mass(303, *[0.0006, 0.0006, 0.0006])
    ops.mass(304, *[0.0005, 0.0005, 0.0005])
    ops.mass(305, *[0.0007, 0.0007, 0.0007])
    ops.mass(306, *[0.0005, 0.0005, 0.0005])
    ops.mass(307, *[0.0009, 0.0009, 0.0009])
    ops.mass(308, *[0.0013, 0.0013, 0.0013])
    ops.mass(309, *[0.0006, 0.0006, 0.0006])
    ops.mass(310, *[0.0004, 0.0004, 0.0004])
    ops.mass(311, *[0.0007, 0.0007, 0.0007])
    ops.mass(312, *[0.0007, 0.0007, 0.0007])
    ops.mass(313, *[0.0007, 0.0007, 0.0007])
    ops.mass(314, *[0.0008, 0.0008, 0.0008])
    ops.mass(315, *[0.0003, 0.0003, 0.0003])
    ops.mass(316, *[0.0008, 0.0008, 0.0008])
    ops.mass(317, *[0.0007, 0.0007, 0.0007])
    ops.mass(318, *[0.0009, 0.0009, 0.0009])
    ops.mass(319, *[0.0007, 0.0007, 0.0007])
    ops.mass(320, *[0.0008, 0.0008, 0.0008])
    ops.mass(321, *[0.0003, 0.0003, 0.0003])
    ops.mass(322, *[0.0007, 0.0007, 0.0007])
    ops.mass(323, *[0.0004, 0.0004, 0.0004])
    ops.mass(324, *[0.0003, 0.0003, 0.0003])
    ops.mass(325, *[0.0007, 0.0007, 0.0007])
    ops.mass(326, *[0.001, 0.001, 0.001])
    ops.mass(327, *[0.0012, 0.0012, 0.0012])
    ops.mass(328, *[0.0015, 0.0015, 0.0015])
    ops.mass(329, *[0.001, 0.001, 0.001])
    ops.mass(330, *[0.0007, 0.0007, 0.0007])
    ops.mass(331, *[0.0008, 0.0008, 0.0008])
    ops.mass(332, *[0.0011, 0.0011, 0.0011])
    ops.mass(333, *[0.0007, 0.0007, 0.0007])
    ops.mass(334, *[0.0008, 0.0008, 0.0008])
    ops.mass(335, *[0.0007, 0.0007, 0.0007])
    ops.mass(336, *[0.0004, 0.0004, 0.0004])
    ops.mass(337, *[0.0004, 0.0004, 0.0004])
    ops.mass(338, *[0.0003, 0.0003, 0.0003])
    ops.mass(339, *[0.0005, 0.0005, 0.0005])
    ops.mass(340, *[0.0004, 0.0004, 0.0004])
    ops.mass(341, *[0.0011, 0.0011, 0.0011])
    ops.mass(342, *[0.0008, 0.0008, 0.0008])
    ops.mass(343, *[0.0009, 0.0009, 0.0009])
    ops.mass(344, *[0.0009, 0.0009, 0.0009])
    ops.mass(345, *[0.0006, 0.0006, 0.0006])
    ops.mass(346, *[0.0005, 0.0005, 0.0005])
    ops.mass(347, *[0.0006, 0.0006, 0.0006])
    ops.mass(348, *[0.0008, 0.0008, 0.0008])
    ops.mass(349, *[0.0004, 0.0004, 0.0004])
    ops.mass(350, *[0.001, 0.001, 0.001])
    ops.mass(351, *[0.0018, 0.0018, 0.0018])
    ops.mass(352, *[0.0008, 0.0008, 0.0008])
    ops.mass(353, *[0.0008, 0.0008, 0.0008])
    ops.mass(354, *[0.0005, 0.0005, 0.0005])
    ops.mass(355, *[0.0006, 0.0006, 0.0006])
    ops.mass(356, *[0.001, 0.001, 0.001])
    ops.mass(357, *[0.0008, 0.0008, 0.0008])
    ops.mass(358, *[0.0016, 0.0016, 0.0016])
    ops.mass(359, *[0.0015, 0.0015, 0.0015])
    ops.mass(360, *[0.0009, 0.0009, 0.0009])
    ops.mass(361, *[0.0013, 0.0013, 0.0013])
    ops.mass(362, *[0.0013, 0.0013, 0.0013])
    ops.mass(363, *[0.0006, 0.0006, 0.0006])
    ops.mass(364, *[0.0008, 0.0008, 0.0008])
    ops.mass(365, *[0.001, 0.001, 0.001])
    ops.mass(366, *[0.0013, 0.0013, 0.0013])
    ops.mass(367, *[0.0016, 0.0016, 0.0016])
    ops.mass(368, *[0.0015, 0.0015, 0.0015])
    ops.mass(369, *[0.0005, 0.0005, 0.0005])
    ops.mass(370, *[0.0008, 0.0008, 0.0008])
    ops.mass(371, *[0.0016, 0.0016, 0.0016])
    ops.mass(372, *[0.0025, 0.0025, 0.0025])
    ops.mass(373, *[0.0019, 0.0019, 0.0019])
    ops.mass(374, *[0.0013, 0.0013, 0.0013])
    ops.mass(375, *[0.0019, 0.0019, 0.0019])
    ops.mass(376, *[0.0014, 0.0014, 0.0014])
    ops.mass(377, *[0.0009, 0.0009, 0.0009])
    ops.mass(378, *[0.0006, 0.0006, 0.0006])
    ops.mass(379, *[0.0004, 0.0004, 0.0004])
    ops.mass(380, *[0.0002, 0.0002, 0.0002])
    ops.mass(381, *[0.0005, 0.0005, 0.0005])
    ops.mass(382, *[0.0008, 0.0008, 0.0008])
    ops.mass(383, *[0.0007, 0.0007, 0.0007])
    ops.mass(384, *[0.0012, 0.0012, 0.0012])
    ops.mass(385, *[0.0008, 0.0008, 0.0008])
    ops.mass(386, *[0.0006, 0.0006, 0.0006])
    ops.mass(387, *[0.0015, 0.0015, 0.0015])
    ops.mass(388, *[0.0013, 0.0013, 0.0013])
    ops.mass(389, *[0.0014, 0.0014, 0.0014])
    ops.mass(390, *[0.0016, 0.0016, 0.0016])
    ops.mass(391, *[0.0014, 0.0014, 0.0014])
    ops.mass(392, *[0.0017, 0.0017, 0.0017])
    ops.mass(393, *[0.0006, 0.0006, 0.0006])
    ops.mass(394, *[0.0031, 0.0031, 0.0031])
    ops.mass(395, *[0.0018, 0.0018, 0.0018])
    ops.mass(396, *[0.0019, 0.0019, 0.0019])
    ops.mass(397, *[0.0017, 0.0017, 0.0017])
    ops.mass(398, *[0.0011, 0.0011, 0.0011])
    ops.mass(399, *[0.0016, 0.0016, 0.0016])
    ops.mass(400, *[0.0011, 0.0011, 0.0011])
    ops.mass(401, *[0.0011, 0.0011, 0.0011])
    ops.mass(402, *[0.0028, 0.0028, 0.0028])
    ops.mass(403, *[0.0026, 0.0026, 0.0026])
    ops.mass(404, *[0.0021, 0.0021, 0.0021])
    ops.mass(405, *[0.0014, 0.0014, 0.0014])
    ops.mass(406, *[0.0007, 0.0007, 0.0007])
    ops.mass(407, *[0.0005, 0.0005, 0.0005])
    ops.mass(408, *[0.0002, 0.0002, 0.0002])
    ops.mass(409, *[0.001, 0.001, 0.001])
    ops.mass(410, *[0.0008, 0.0008, 0.0008])
    ops.mass(411, *[0.0004, 0.0004, 0.0004])
    ops.mass(412, *[0.0011, 0.0011, 0.0011])
    ops.mass(413, *[0.0005, 0.0005, 0.0005])
    ops.mass(414, *[0.0012, 0.0012, 0.0012])
    ops.mass(415, *[0.0008, 0.0008, 0.0008])
    ops.mass(416, *[0.0015, 0.0015, 0.0015])
    ops.mass(417, *[0.0019, 0.0019, 0.0019])
    ops.mass(418, *[0.0018, 0.0018, 0.0018])
    ops.mass(419, *[0.0006, 0.0006, 0.0006])
    ops.mass(420, *[0.0008, 0.0008, 0.0008])
    ops.mass(421, *[0.0008, 0.0008, 0.0008])
    ops.mass(422, *[0.0006, 0.0006, 0.0006])
    ops.mass(423, *[0.0008, 0.0008, 0.0008])
    ops.mass(424, *[0.0007, 0.0007, 0.0007])
    ops.mass(431, *[0.0018, 0.0018, 0.0018])
    ops.mass(432, *[0.0013, 0.0013, 0.0013])
    ops.mass(433, *[0.0013, 0.0013, 0.0013])
    ops.mass(434, *[0.0003, 0.0003, 0.0003])
    ops.mass(435, *[0.0003, 0.0003, 0.0003])
    ops.mass(436, *[0.0008, 0.0008, 0.0008])
    ops.mass(437, *[0.0007, 0.0007, 0.0007])
    ops.mass(438, *[0.0003, 0.0003, 0.0003])
    ops.mass(439, *[0.0004, 0.0004, 0.0004])
    ops.mass(440, *[0.0009, 0.0009, 0.0009])
    ops.mass(441, *[0.0003, 0.0003, 0.0003])
    ops.mass(442, *[0.0006, 0.0006, 0.0006])
    ops.mass(443, *[0.0004, 0.0004, 0.0004])
    ops.mass(444, *[0.0007, 0.0007, 0.0007])
    ops.mass(445, *[0.0005, 0.0005, 0.0005])
    ops.mass(446, *[0.0004, 0.0004, 0.0004])
    ops.mass(447, *[0.0005, 0.0005, 0.0005])
    ops.mass(448, *[0.0005, 0.0005, 0.0005])
    ops.mass(449, *[0.0003, 0.0003, 0.0003])
    ops.mass(450, *[0.0002, 0.0002, 0.0002])
    ops.mass(451, *[0.0006, 0.0006, 0.0006])
    ops.mass(452, *[0.0004, 0.0004, 0.0004])
    ops.mass(453, *[0.0003, 0.0003, 0.0003])
    ops.mass(454, *[0.0007, 0.0007, 0.0007])
    ops.mass(455, *[0.0005, 0.0005, 0.0005])
    ops.mass(456, *[0.0005, 0.0005, 0.0005])
    ops.mass(457, *[0.0005, 0.0005, 0.0005])
    ops.mass(458, *[0.0006, 0.0006, 0.0006])
    ops.mass(459, *[0.0007, 0.0007, 0.0007])
    ops.mass(460, *[0.0002, 0.0002, 0.0002])
    ops.mass(461, *[0.0004, 0.0004, 0.0004])
    ops.mass(462, *[0.0007, 0.0007, 0.0007])
    ops.mass(463, *[0.0007, 0.0007, 0.0007])
    ops.mass(464, *[0.0003, 0.0003, 0.0003])
    ops.mass(465, *[0.0005, 0.0005, 0.0005])
    ops.mass(466, *[0.0004, 0.0004, 0.0004])
    ops.mass(467, *[0.0004, 0.0004, 0.0004])
    ops.mass(468, *[0.0006, 0.0006, 0.0006])
    ops.mass(469, *[0.0005, 0.0005, 0.0005])
    ops.mass(470, *[0.0005, 0.0005, 0.0005])
    ops.mass(471, *[0.0007, 0.0007, 0.0007])
    ops.mass(472, *[0.0008, 0.0008, 0.0008])
    ops.mass(473, *[0.0005, 0.0005, 0.0005])
    ops.mass(474, *[0.0002, 0.0002, 0.0002])
    ops.mass(475, *[0.0001, 0.0001, 0.0001])
    ops.mass(476, *[0.0003, 0.0003, 0.0003])
    ops.mass(477, *[0.0007, 0.0007, 0.0007])
    ops.mass(478, *[0.0006, 0.0006, 0.0006])
    ops.mass(479, *[0.0003, 0.0003, 0.0003])
    ops.mass(480, *[0.0004, 0.0004, 0.0004])
    ops.mass(481, *[0.0008, 0.0008, 0.0008])
    ops.mass(482, *[0.0019, 0.0019, 0.0019])
    ops.mass(483, *[0.0009, 0.0009, 0.0009])
    ops.mass(484, *[0.0007, 0.0007, 0.0007])
    ops.mass(485, *[0.0012, 0.0012, 0.0012])
    ops.mass(486, *[0.0017, 0.0017, 0.0017])
    ops.mass(487, *[0.001, 0.001, 0.001])
    ops.mass(488, *[0.0018, 0.0018, 0.0018])
    ops.mass(489, *[0.0019, 0.0019, 0.0019])
    ops.mass(490, *[0.001, 0.001, 0.001])
    ops.mass(491, *[0.0043, 0.0043, 0.0043])
    ops.mass(492, *[0.0019, 0.0019, 0.0019])
    ops.mass(493, *[0.0021, 0.0021, 0.0021])
    ops.mass(494, *[0.0007, 0.0007, 0.0007])
    ops.mass(509, *[0.0027, 0.0027, 0.0027])
    ops.mass(510, *[0.0007, 0.0007, 0.0007])
    ops.mass(511, *[0.0018, 0.0018, 0.0018])
    ops.mass(512, *[0.0072, 0.0072, 0.0072])
    ops.mass(513, *[0.0129, 0.0129, 0.0129])
    ops.mass(514, *[0.002, 0.002, 0.002])
    ops.mass(515, *[0.0017, 0.0017, 0.0017])
    ops.mass(516, *[0.0031, 0.0031, 0.0031])
    ops.mass(517, *[0.0124, 0.0124, 0.0124])
    ops.mass(518, *[0.0102, 0.0102, 0.0102])
    ops.mass(519, *[0.0077, 0.0077, 0.0077])
    ops.mass(520, *[0.0053, 0.0053, 0.0053])
    ops.mass(521, *[0.0079, 0.0079, 0.0079])
    ops.mass(522, *[0.0021, 0.0021, 0.0021])
    ops.mass(523, *[0.0013, 0.0013, 0.0013])
    ops.mass(524, *[0.0013, 0.0013, 0.0013])
    ops.mass(525, *[0.0027, 0.0027, 0.0027])
    ops.mass(526, *[0.001, 0.001, 0.001])
    ops.mass(527, *[0.0009, 0.0009, 0.0009])
    ops.mass(528, *[0.0005, 0.0005, 0.0005])
    ops.mass(529, *[0.001, 0.001, 0.001])
    ops.mass(530, *[0.0008, 0.0008, 0.0008])
    ops.mass(531, *[0.0008, 0.0008, 0.0008])
    ops.mass(532, *[0.0007, 0.0007, 0.0007])
    ops.mass(533, *[0.0005, 0.0005, 0.0005])
    ops.mass(534, *[0.0007, 0.0007, 0.0007])
    ops.mass(535, *[0.0007, 0.0007, 0.0007])
    ops.mass(536, *[0.0005, 0.0005, 0.0005])
    ops.mass(537, *[0.0006, 0.0006, 0.0006])
    ops.mass(538, *[0.0006, 0.0006, 0.0006])
    ops.mass(545, *[0.0024, 0.0024, 0.0024])
    ops.mass(546, *[0.0017, 0.0017, 0.0017])
    ops.mass(547, *[0.0017, 0.0017, 0.0017])
    ops.mass(548, *[0.0021, 0.0021, 0.0021])
    ops.mass(549, *[0.002, 0.002, 0.002])
    ops.mass(550, *[0.0022, 0.0022, 0.0022])
    ops.mass(551, *[0.0016, 0.0016, 0.0016])
    ops.mass(601, *[0.0016, 0.0016, 0.0016])
    ops.mass(602, *[0.0016, 0.0016, 0.0016])
    ops.mass(603, *[0.0011, 0.0011, 0.0011])
    ops.mass(604, *[0.0022, 0.0022, 0.0022])
    ops.mass(605, *[0.0011, 0.0011, 0.0011])
    ops.mass(606, *[0.003, 0.003, 0.003])
    ops.mass(607, *[0.002, 0.002, 0.002])
    ops.mass(608, *[0.0013, 0.0013, 0.0013])
    ops.mass(609, *[0.0008, 0.0008, 0.0008])
    ops.mass(610, *[0.0011, 0.0011, 0.0011])
    ops.mass(611, *[0.001, 0.001, 0.001])
    ops.mass(612, *[0.001, 0.001, 0.001])
    ops.mass(613, *[0.0005, 0.0005, 0.0005])
    ops.mass(614, *[0.0016, 0.0016, 0.0016])
    ops.mass(615, *[0.0013, 0.0013, 0.0013])
    ops.mass(616, *[0.0013, 0.0013, 0.0013])
    ops.mass(617, *[0.0013, 0.0013, 0.0013])
    ops.mass(618, *[0.0014, 0.0014, 0.0014])
    ops.mass(619, *[0.0007, 0.0007, 0.0007])
    ops.mass(620, *[0.0011, 0.0011, 0.0011])
    ops.mass(621, *[0.0023, 0.0023, 0.0023])
    ops.mass(622, *[0.0009, 0.0009, 0.0009])
    ops.mass(623, *[0.0014, 0.0014, 0.0014])
    ops.mass(624, *[0.0017, 0.0017, 0.0017])
    ops.mass(625, *[0.0014, 0.0014, 0.0014])
    ops.mass(626, *[0.001, 0.001, 0.001])
    ops.mass(627, *[0.0015, 0.0015, 0.0015])
    ops.mass(628, *[0.0012, 0.0012, 0.0012])
    ops.mass(629, *[0.0017, 0.0017, 0.0017])
    ops.mass(630, *[0.0018, 0.0018, 0.0018])
    ops.mass(631, *[0.0014, 0.0014, 0.0014])
    ops.mass(632, *[0.0008, 0.0008, 0.0008])
    ops.mass(633, *[0.0005, 0.0005, 0.0005])
    ops.mass(634, *[0.0015, 0.0015, 0.0015])
    ops.mass(645, *[0.0014, 0.0014, 0.0014])
    ops.mass(646, *[0.0013, 0.0013, 0.0013])
    ops.mass(647, *[0.0013, 0.0013, 0.0013])
    ops.mass(648, *[0.0014, 0.0014, 0.0014])
    ops.mass(649, *[0.0014, 0.0014, 0.0014])
    ops.mass(650, *[0.0003, 0.0003, 0.0003])
    ops.mass(651, *[0.0005, 0.0005, 0.0005])
    ops.mass(652, *[0.0007, 0.0007, 0.0007])
    ops.mass(653, *[0.001, 0.001, 0.001])
    ops.mass(654, *[0.0002, 0.0002, 0.0002])
    ops.mass(655, *[0.0004, 0.0004, 0.0004])
    ops.mass(656, *[0.0008, 0.0008, 0.0008])
    ops.mass(657, *[0.0007, 0.0007, 0.0007])
    ops.mass(658, *[0.0008, 0.0008, 0.0008])
    ops.mass(659, *[0.0005, 0.0005, 0.0005])
    ops.mass(660, *[0.0004, 0.0004, 0.0004])
    ops.mass(661, *[0.0007, 0.0007, 0.0007])
    ops.mass(662, *[0.0006, 0.0006, 0.0006])
    ops.mass(663, *[0.0009, 0.0009, 0.0009])
    ops.mass(664, *[0.0007, 0.0007, 0.0007])
    ops.mass(665, *[0.0006, 0.0006, 0.0006])
    ops.mass(666, *[0.0006, 0.0006, 0.0006])
    ops.mass(667, *[0.0009, 0.0009, 0.0009])
    ops.mass(668, *[0.0016, 0.0016, 0.0016])
    ops.mass(669, *[0.0011, 0.0011, 0.0011])
    ops.mass(670, *[0.0009, 0.0009, 0.0009])
    ops.mass(671, *[0.0015, 0.0015, 0.0015])
    ops.mass(672, *[0.0012, 0.0012, 0.0012])
    ops.mass(673, *[0.0008, 0.0008, 0.0008])
    ops.mass(674, *[0.0014, 0.0014, 0.0014])
    ops.mass(675, *[0.0012, 0.0012, 0.0012])
    ops.mass(676, *[0.0006, 0.0006, 0.0006])
    ops.mass(677, *[0.0018, 0.0018, 0.0018])
    ops.mass(678, *[0.0014, 0.0014, 0.0014])
    ops.mass(679, *[0.0007, 0.0007, 0.0007])
    ops.mass(680, *[0.001, 0.001, 0.001])
    ops.mass(681, *[0.0014, 0.0014, 0.0014])
    ops.mass(682, *[0.0005, 0.0005, 0.0005])
    ops.mass(683, *[0.001, 0.001, 0.001])
    ops.mass(684, *[0.0015, 0.0015, 0.0015])
    ops.mass(702, *[0.001, 0.001, 0.001])
    ops.mass(703, *[0.0007, 0.0007, 0.0007])
    ops.mass(704, *[0.0008, 0.0008, 0.0008])
    ops.mass(705, *[0.0008, 0.0008, 0.0008])
    ops.mass(706, *[0.001, 0.001, 0.001])
    ops.mass(707, *[0.0012, 0.0012, 0.0012])
    ops.mass(708, *[0.0015, 0.0015, 0.0015])
    ops.mass(709, *[0.0014, 0.0014, 0.0014])
    ops.mass(710, *[0.0011, 0.0011, 0.0011])
    ops.mass(711, *[0.0012, 0.0012, 0.0012])
    ops.mass(712, *[0.0012, 0.0012, 0.0012])
    ops.mass(713, *[0.0006, 0.0006, 0.0006])
    ops.mass(714, *[0.0007, 0.0007, 0.0007])
    ops.mass(715, *[0.0006, 0.0006, 0.0006])
    ops.mass(716, *[0.0006, 0.0006, 0.0006])
    ops.mass(717, *[0.0005, 0.0005, 0.0005])
    ops.mass(718, *[0.0007, 0.0007, 0.0007])
    ops.mass(719, *[0.0003, 0.0003, 0.0003])
    ops.mass(720, *[0.0007, 0.0007, 0.0007])
    ops.mass(721, *[0.0006, 0.0006, 0.0006])
    ops.mass(722, *[0.0004, 0.0004, 0.0004])
    ops.mass(723, *[0.0011, 0.0011, 0.0011])
    ops.mass(724, *[0.0006, 0.0006, 0.0006])
    ops.mass(725, *[0.0005, 0.0005, 0.0005])
    ops.mass(726, *[0.0006, 0.0006, 0.0006])
    ops.mass(727, *[0.0006, 0.0006, 0.0006])
    ops.mass(728, *[0.0003, 0.0003, 0.0003])
    ops.mass(729, *[0.0008, 0.0008, 0.0008])
    ops.mass(730, *[0.0007, 0.0007, 0.0007])
    ops.mass(731, *[0.0006, 0.0006, 0.0006])
    ops.mass(732, *[0.0011, 0.0011, 0.0011])
    ops.mass(733, *[0.0005, 0.0005, 0.0005])
    ops.mass(734, *[0.0007, 0.0007, 0.0007])
    ops.mass(735, *[0.0008, 0.0008, 0.0008])
    ops.mass(736, *[0.0003, 0.0003, 0.0003])
    ops.mass(737, *[0.0008, 0.0008, 0.0008])
    ops.mass(738, *[0.001, 0.001, 0.001])
    ops.mass(739, *[0.0005, 0.0005, 0.0005])
    ops.mass(740, *[0.001, 0.001, 0.001])
    ops.mass(741, *[0.0008, 0.0008, 0.0008])
    ops.mass(742, *[0.0009, 0.0009, 0.0009])
    ops.mass(743, *[0.0009, 0.0009, 0.0009])
    ops.mass(744, *[0.0012, 0.0012, 0.0012])
    ops.mass(745, *[0.0009, 0.0009, 0.0009])
    ops.mass(746, *[0.001, 0.001, 0.001])
    ops.mass(747, *[0.0005, 0.0005, 0.0005])
    ops.mass(748, *[0.0012, 0.0012, 0.0012])
    ops.mass(749, *[0.0007, 0.0007, 0.0007])
    ops.mass(750, *[0.0016, 0.0016, 0.0016])
    ops.mass(751, *[0.0013, 0.0013, 0.0013])
    ops.mass(752, *[0.0011, 0.0011, 0.0011])
    ops.mass(753, *[0.0001, 0.0001, 0.0001])
    ops.mass(754, *[0.0006, 0.0006, 0.0006])
    ops.mass(755, *[0.0005, 0.0005, 0.0005])
    ops.mass(756, *[0.0009, 0.0009, 0.0009])
    ops.mass(757, *[0.0009, 0.0009, 0.0009])
    ops.mass(758, *[0.0006, 0.0006, 0.0006])
    ops.mass(759, *[0.0006, 0.0006, 0.0006])
    ops.mass(760, *[0.0009, 0.0009, 0.0009])
    ops.mass(761, *[0.0008, 0.0008, 0.0008])
    ops.mass(762, *[0.0008, 0.0008, 0.0008])
    ops.mass(763, *[0.0008, 0.0008, 0.0008])
    ops.mass(764, *[0.0005, 0.0005, 0.0005])
    ops.mass(765, *[0.0003, 0.0003, 0.0003])
    ops.mass(766, *[0.0005, 0.0005, 0.0005])
    ops.mass(767, *[0.0007, 0.0007, 0.0007])
    ops.mass(768, *[0.0005, 0.0005, 0.0005])
    ops.mass(769, *[0.0007, 0.0007, 0.0007])
    ops.mass(770, *[0.0009, 0.0009, 0.0009])
    ops.mass(771, *[0.0007, 0.0007, 0.0007])
    ops.mass(772, *[0.0002, 0.0002, 0.0002])
    ops.mass(1121, *[0.0016, 0.0016, 0.0016])
    ops.mass(1122, *[0.0029, 0.0029, 0.0029])
    ops.mass(1123, *[0.0042, 0.0042, 0.0042])
    ops.mass(1124, *[0.0033, 0.0033, 0.0033])
    ops.mass(1125, *[0.0021, 0.0021, 0.0021])
    ops.mass(1126, *[0.0028, 0.0028, 0.0028])
    ops.mass(1127, *[0.0027, 0.0027, 0.0027])
    ops.mass(1128, *[0.0011, 0.0011, 0.0011])
    ops.mass(1129, *[0.0032, 0.0032, 0.0032])
    ops.mass(1130, *[0.0043, 0.0043, 0.0043])
    ops.mass(1131, *[0.0017, 0.0017, 0.0017])
    ops.mass(1132, *[0.0019, 0.0019, 0.0019])
    ops.mass(1133, *[0.0019, 0.0019, 0.0019])
    ops.mass(1134, *[0.0037, 0.0037, 0.0037])
    ops.mass(1135, *[0.0044, 0.0044, 0.0044])
    ops.mass(1136, *[0.0018, 0.0018, 0.0018])
    ops.mass(1137, *[0.0037, 0.0037, 0.0037])
    ops.mass(1138, *[0.0025, 0.0025, 0.0025])
    ops.mass(1139, *[0.0019, 0.0019, 0.0019])
    ops.mass(1140, *[0.0035, 0.0035, 0.0035])
    ops.mass(1141, *[0.0021, 0.0021, 0.0021])
    ops.mass(1142, *[0.0012, 0.0012, 0.0012])
    ops.mass(1143, *[0.0013, 0.0013, 0.0013])
    ops.mass(1144, *[0.0017, 0.0017, 0.0017])
    ops.mass(1145, *[0.0009, 0.0009, 0.0009])
    ops.mass(1146, *[0.0033, 0.0033, 0.0033])
    ops.mass(1147, *[0.0024, 0.0024, 0.0024])
    ops.mass(1148, *[0.0014, 0.0014, 0.0014])
    ops.mass(1149, *[0.0034, 0.0034, 0.0034])
    ops.mass(1150, *[0.0112, 0.0112, 0.0112])
    ops.mass(1151, *[0.0017, 0.0017, 0.0017])
    ops.mass(1152, *[0.0025, 0.0025, 0.0025])
    ops.mass(1153, *[0.003, 0.003, 0.003])
    ops.mass(1154, *[0.0019, 0.0019, 0.0019])
    ops.mass(1155, *[0.0059, 0.0059, 0.0059])
    ops.mass(1156, *[0.0017, 0.0017, 0.0017])
    ops.mass(1157, *[0.0034, 0.0034, 0.0034])
    ops.mass(1158, *[0.0053, 0.0053, 0.0053])
    ops.mass(1159, *[0.0031, 0.0031, 0.0031])
    ops.mass(1160, *[0.0016, 0.0016, 0.0016])
    ops.mass(1161, *[0.001, 0.001, 0.001])
    ops.mass(1162, *[0.0018, 0.0018, 0.0018])
    ops.mass(1163, *[0.002, 0.002, 0.002])
    ops.mass(1164, *[0.0016, 0.0016, 0.0016])
    ops.mass(1165, *[0.0019, 0.0019, 0.0019])
    ops.mass(1166, *[0.0015, 0.0015, 0.0015])
    ops.mass(1167, *[0.0005, 0.0005, 0.0005])
    ops.mass(1168, *[0.0005, 0.0005, 0.0005])
    ops.mass(1169, *[0.0013, 0.0013, 0.0013])
    ops.mass(1170, *[0.0043, 0.0043, 0.0043])
    ops.mass(1171, *[0.001, 0.001, 0.001])
    ops.mass(1172, *[0.002, 0.002, 0.002])
    ops.mass(1173, *[0.0005, 0.0005, 0.0005])
    ops.mass(1174, *[0.0016, 0.0016, 0.0016])
    ops.mass(1175, *[0.0027, 0.0027, 0.0027])
    ops.mass(1176, *[0.003, 0.003, 0.003])
    ops.mass(1177, *[0.0026, 0.0026, 0.0026])
    ops.mass(1178, *[0.0013, 0.0013, 0.0013])
    ops.mass(1179, *[0.0017, 0.0017, 0.0017])
    ops.mass(1180, *[0.0011, 0.0011, 0.0011])
    ops.mass(1181, *[0.0021, 0.0021, 0.0021])
    ops.mass(1182, *[0.0033, 0.0033, 0.0033])
    ops.mass(1183, *[0.0026, 0.0026, 0.0026])
    ops.mass(1184, *[0.0022, 0.0022, 0.0022])
    ops.mass(1185, *[0.0024, 0.0024, 0.0024])
    ops.mass(1186, *[0.0028, 0.0028, 0.0028])
    ops.mass(1187, *[0.0017, 0.0017, 0.0017])
    ops.mass(1188, *[0.0016, 0.0016, 0.0016])
    ops.mass(1189, *[0.0017, 0.0017, 0.0017])
    ops.mass(1190, *[0.0038, 0.0038, 0.0038])
    ops.mass(1191, *[0.0045, 0.0045, 0.0045])
    ops.mass(1192, *[0.0011, 0.0011, 0.0011])
    ops.mass(1193, *[0.0017, 0.0017, 0.0017])
    ops.mass(1194, *[0.0028, 0.0028, 0.0028])
    ops.mass(1195, *[0.0036, 0.0036, 0.0036])
    ops.mass(1196, *[0.0018, 0.0018, 0.0018])
    ops.mass(1197, *[0.0023, 0.0023, 0.0023])
    ops.mass(1198, *[0.0022, 0.0022, 0.0022])
    ops.mass(1199, *[0.0013, 0.0013, 0.0013])
    ops.mass(1200, *[0.0018, 0.0018, 0.0018])
    ops.mass(1201, *[0.0029, 0.0029, 0.0029])
    ops.mass(1202, *[0.0046, 0.0046, 0.0046])
    ops.mass(1203, *[0.0032, 0.0032, 0.0032])
    ops.mass(1204, *[0.004, 0.004, 0.004])
    ops.mass(1205, *[0.0017, 0.0017, 0.0017])
    ops.mass(1206, *[0.0043, 0.0043, 0.0043])
    ops.mass(1207, *[0.0011, 0.0011, 0.0011])
    ops.mass(1208, *[0.0036, 0.0036, 0.0036])
    ops.mass(1209, *[0.0023, 0.0023, 0.0023])
    ops.mass(1210, *[0.0024, 0.0024, 0.0024])
    ops.mass(1211, *[0.0019, 0.0019, 0.0019])
    ops.mass(1212, *[0.0017, 0.0017, 0.0017])
    ops.mass(1213, *[0.0016, 0.0016, 0.0016])
    ops.mass(1214, *[0.0026, 0.0026, 0.0026])
    ops.mass(1215, *[0.0041, 0.0041, 0.0041])
    ops.mass(1216, *[0.0035, 0.0035, 0.0035])
    ops.mass(1217, *[0.0023, 0.0023, 0.0023])
    ops.mass(1218, *[0.002, 0.002, 0.002])
    ops.mass(1219, *[0.0015, 0.0015, 0.0015])
    ops.mass(1220, *[0.0027, 0.0027, 0.0027])
    ops.mass(1221, *[0.005, 0.005, 0.005])
    ops.mass(1222, *[0.0026, 0.0026, 0.0026])
    ops.mass(1223, *[0.0025, 0.0025, 0.0025])
    ops.mass(1224, *[0.0028, 0.0028, 0.0028])
    ops.mass(1225, *[0.0012, 0.0012, 0.0012])
    ops.mass(1226, *[0.0017, 0.0017, 0.0017])
    ops.mass(1227, *[0.0022, 0.0022, 0.0022])
    ops.mass(1228, *[0.0031, 0.0031, 0.0031])
    ops.mass(1229, *[0.0053, 0.0053, 0.0053])
    ops.mass(1230, *[0.0015, 0.0015, 0.0015])
    ops.mass(1231, *[0.0036, 0.0036, 0.0036])
    ops.mass(1232, *[0.0036, 0.0036, 0.0036])
    ops.mass(1233, *[0.0016, 0.0016, 0.0016])
    ops.mass(1234, *[0.001, 0.001, 0.001])
    ops.mass(1235, *[0.0022, 0.0022, 0.0022])
    ops.mass(1236, *[0.0018, 0.0018, 0.0018])
    ops.mass(1237, *[0.0011, 0.0011, 0.0011])
    ops.mass(1238, *[0.0034, 0.0034, 0.0034])
    ops.mass(1239, *[0.0006, 0.0006, 0.0006])
    ops.mass(1240, *[0.0023, 0.0023, 0.0023])
    ops.mass(1241, *[0.0016, 0.0016, 0.0016])
    ops.mass(1242, *[0.002, 0.002, 0.002])
    ops.mass(1243, *[0.0009, 0.0009, 0.0009])
    ops.mass(1244, *[0.0011, 0.0011, 0.0011])
    ops.mass(1245, *[0.002, 0.002, 0.002])
    ops.mass(1246, *[0.0042, 0.0042, 0.0042])
    ops.mass(1247, *[0.0016, 0.0016, 0.0016])
    ops.mass(1248, *[0.0014, 0.0014, 0.0014])
    ops.mass(1249, *[0.0022, 0.0022, 0.0022])
    ops.mass(1250, *[0.0008, 0.0008, 0.0008])
    ops.mass(1251, *[0.0022, 0.0022, 0.0022])
    ops.mass(1252, *[0.0007, 0.0007, 0.0007])
    ops.mass(1253, *[0.0012, 0.0012, 0.0012])
    ops.mass(1254, *[0.0003, 0.0003, 0.0003])
    ops.mass(1255, *[0.0017, 0.0017, 0.0017])
    ops.mass(1256, *[0.0015, 0.0015, 0.0015])
    ops.mass(1257, *[0.0016, 0.0016, 0.0016])
    ops.mass(1258, *[0.0026, 0.0026, 0.0026])
    ops.mass(1259, *[0.0001, 0.0001, 0.0001])
    ops.mass(1260, *[0.0006, 0.0006, 0.0006])
    ops.mass(1261, *[0.0002, 0.0002, 0.0002])
    ops.mass(1262, *[0.0008, 0.0008, 0.0008])
    ops.mass(1263, *[0.0009, 0.0009, 0.0009])
    ops.mass(1264, *[0.0003, 0.0003, 0.0003])
    ops.mass(1265, *[0.0006, 0.0006, 0.0006])
    ops.mass(1266, *[0.0004, 0.0004, 0.0004])
    ops.mass(1267, *[0.0017, 0.0017, 0.0017])
    ops.mass(1268, *[0.0004, 0.0004, 0.0004])
    ops.mass(1269, *[0.0009, 0.0009, 0.0009])
    ops.mass(1270, *[0.0014, 0.0014, 0.0014])
    ops.mass(1271, *[0.001, 0.001, 0.001])
    ops.mass(1272, *[0.0016, 0.0016, 0.0016])
    ops.mass(1273, *[0.0007, 0.0007, 0.0007])
    ops.mass(1274, *[0.0007, 0.0007, 0.0007])
    ops.mass(1275, *[0.0013, 0.0013, 0.0013])
    ops.mass(1276, *[0.001, 0.001, 0.001])
    ops.mass(1277, *[0.0015, 0.0015, 0.0015])
    ops.mass(1278, *[0.0009, 0.0009, 0.0009])
    ops.mass(1279, *[0.001, 0.001, 0.001])
    ops.mass(1280, *[0.0011, 0.0011, 0.0011])
    ops.mass(1281, *[0.0016, 0.0016, 0.0016])
    ops.mass(1282, *[0.002, 0.002, 0.002])
    ops.mass(1283, *[0.0013, 0.0013, 0.0013])
    ops.mass(1284, *[0.0009, 0.0009, 0.0009])
    ops.mass(1285, *[0.0021, 0.0021, 0.0021])
    ops.mass(1286, *[0.0014, 0.0014, 0.0014])
    ops.mass(1287, *[0.0021, 0.0021, 0.0021])
    ops.mass(1288, *[0.0012, 0.0012, 0.0012])
    ops.mass(1289, *[0.0011, 0.0011, 0.0011])
    ops.mass(1290, *[0.0013, 0.0013, 0.0013])
    ops.mass(1291, *[0.0017, 0.0017, 0.0017])
    ops.mass(1292, *[0.0016, 0.0016, 0.0016])
    ops.mass(1293, *[0.0015, 0.0015, 0.0015])
    ops.mass(1294, *[0.0009, 0.0009, 0.0009])
    ops.mass(1295, *[0.0015, 0.0015, 0.0015])
    ops.mass(1296, *[0.0019, 0.0019, 0.0019])
    ops.mass(1297, *[0.0012, 0.0012, 0.0012])
    ops.mass(1298, *[0.0019, 0.0019, 0.0019])
    ops.mass(1299, *[0.0013, 0.0013, 0.0013])
    ops.mass(1300, *[0.0044, 0.0044, 0.0044])
    ops.mass(1301, *[0.0022, 0.0022, 0.0022])
    ops.mass(1302, *[0.0013, 0.0013, 0.0013])
    ops.mass(1303, *[0.0021, 0.0021, 0.0021])
    ops.mass(1304, *[0.0009, 0.0009, 0.0009])
    ops.mass(1305, *[0.0013, 0.0013, 0.0013])
    ops.mass(1306, *[0.0025, 0.0025, 0.0025])
    ops.mass(1307, *[0.0009, 0.0009, 0.0009])
    ops.mass(1308, *[0.0012, 0.0012, 0.0012])
    ops.mass(1309, *[0.0022, 0.0022, 0.0022])
    ops.mass(1310, *[0.0015, 0.0015, 0.0015])
    ops.mass(1311, *[0.0017, 0.0017, 0.0017])
    ops.mass(1312, *[0.0032, 0.0032, 0.0032])
    ops.mass(1313, *[0.0021, 0.0021, 0.0021])
    ops.mass(1314, *[0.0008, 0.0008, 0.0008])
    ops.mass(1315, *[0.0011, 0.0011, 0.0011])
    ops.mass(1316, *[0.0015, 0.0015, 0.0015])
    ops.mass(1317, *[0.0013, 0.0013, 0.0013])
    ops.mass(1318, *[0.0016, 0.0016, 0.0016])
    ops.mass(1319, *[0.0018, 0.0018, 0.0018])
    ops.mass(1320, *[0.0009, 0.0009, 0.0009])
    ops.mass(1321, *[0.0012, 0.0012, 0.0012])
    ops.mass(1322, *[0.002, 0.002, 0.002])
    ops.mass(1323, *[0.001, 0.001, 0.001])
    ops.mass(1324, *[0.0018, 0.0018, 0.0018])
    ops.mass(1325, *[0.0018, 0.0018, 0.0018])
    ops.mass(1326, *[0.0014, 0.0014, 0.0014])
    ops.mass(1327, *[0.002, 0.002, 0.002])
    ops.mass(1328, *[0.0011, 0.0011, 0.0011])
    ops.mass(1329, *[0.0024, 0.0024, 0.0024])
    ops.mass(1330, *[0.0012, 0.0012, 0.0012])
    ops.mass(1331, *[0.0016, 0.0016, 0.0016])
    ops.mass(1332, *[0.0018, 0.0018, 0.0018])
    ops.mass(1333, *[0.0016, 0.0016, 0.0016])
    ops.mass(1334, *[0.0012, 0.0012, 0.0012])
    ops.mass(1335, *[0.0016, 0.0016, 0.0016])
    ops.mass(1336, *[0.0016, 0.0016, 0.0016])
    ops.mass(1337, *[0.0016, 0.0016, 0.0016])
    ops.mass(1338, *[0.0026, 0.0026, 0.0026])
    ops.mass(1339, *[0.0017, 0.0017, 0.0017])
    ops.mass(1340, *[0.0021, 0.0021, 0.0021])
    ops.mass(1341, *[0.0021, 0.0021, 0.0021])
    ops.mass(1342, *[0.0024, 0.0024, 0.0024])
    ops.mass(1343, *[0.0012, 0.0012, 0.0012])
    ops.mass(1344, *[0.0019, 0.0019, 0.0019])
    ops.mass(1345, *[0.0016, 0.0016, 0.0016])
    ops.mass(1346, *[0.0013, 0.0013, 0.0013])
    ops.mass(1347, *[0.002, 0.002, 0.002])
    ops.mass(1348, *[0.0016, 0.0016, 0.0016])
    ops.mass(1349, *[0.0019, 0.0019, 0.0019])
    ops.mass(1350, *[0.0017, 0.0017, 0.0017])
    ops.mass(1351, *[0.0011, 0.0011, 0.0011])
    ops.mass(1352, *[0.0046, 0.0046, 0.0046])
    ops.mass(1353, *[0.001, 0.001, 0.001])
    ops.mass(1354, *[0.0019, 0.0019, 0.0019])
    ops.mass(1355, *[0.0015, 0.0015, 0.0015])
    ops.mass(1356, *[0.0016, 0.0016, 0.0016])
    ops.mass(1357, *[0.0022, 0.0022, 0.0022])
    ops.mass(1358, *[0.0013, 0.0013, 0.0013])
    ops.mass(1359, *[0.0016, 0.0016, 0.0016])
    ops.mass(1360, *[0.0012, 0.0012, 0.0012])
    ops.mass(1361, *[0.0014, 0.0014, 0.0014])
    ops.mass(1362, *[0.0025, 0.0025, 0.0025])
    ops.mass(1363, *[0.0021, 0.0021, 0.0021])
    ops.mass(1364, *[0.0012, 0.0012, 0.0012])
    ops.mass(1365, *[0.0012, 0.0012, 0.0012])
    ops.mass(1366, *[0.0018, 0.0018, 0.0018])
    ops.mass(1367, *[0.0012, 0.0012, 0.0012])
    ops.mass(1368, *[0.0019, 0.0019, 0.0019])
    ops.mass(1369, *[0.0012, 0.0012, 0.0012])
    ops.mass(1370, *[0.0014, 0.0014, 0.0014])
    ops.mass(1371, *[0.0013, 0.0013, 0.0013])
    ops.mass(1372, *[0.002, 0.002, 0.002])
    ops.mass(1373, *[0.0028, 0.0028, 0.0028])
    ops.mass(1374, *[0.0014, 0.0014, 0.0014])
    ops.mass(1375, *[0.0022, 0.0022, 0.0022])
    ops.mass(1376, *[0.0013, 0.0013, 0.0013])
    ops.mass(1377, *[0.0016, 0.0016, 0.0016])
    ops.mass(1378, *[0.0017, 0.0017, 0.0017])
    ops.mass(1379, *[0.0028, 0.0028, 0.0028])
    ops.mass(1380, *[0.0044, 0.0044, 0.0044])
    ops.mass(1381, *[0.0021, 0.0021, 0.0021])
    ops.mass(1382, *[0.0023, 0.0023, 0.0023])
    ops.mass(1383, *[0.0015, 0.0015, 0.0015])
    ops.mass(1384, *[0.0016, 0.0016, 0.0016])
    ops.mass(1385, *[0.0023, 0.0023, 0.0023])
    ops.mass(1386, *[0.0017, 0.0017, 0.0017])
    ops.mass(1387, *[0.0023, 0.0023, 0.0023])
    ops.mass(1388, *[0.0023, 0.0023, 0.0023])
    ops.mass(1389, *[0.002, 0.002, 0.002])
    ops.mass(1390, *[0.0025, 0.0025, 0.0025])
    ops.mass(1391, *[0.0023, 0.0023, 0.0023])
    ops.mass(1392, *[0.0016, 0.0016, 0.0016])
    ops.mass(1393, *[0.0013, 0.0013, 0.0013])
    ops.mass(1394, *[0.0018, 0.0018, 0.0018])
    ops.mass(1395, *[0.0021, 0.0021, 0.0021])
    ops.mass(1396, *[0.002, 0.002, 0.002])
    ops.mass(1397, *[0.0022, 0.0022, 0.0022])
    ops.mass(1398, *[0.003, 0.003, 0.003])
    ops.mass(1399, *[0.0015, 0.0015, 0.0015])
    ops.mass(1400, *[0.0027, 0.0027, 0.0027])
    ops.mass(1401, *[0.0012, 0.0012, 0.0012])
    ops.mass(1402, *[0.0028, 0.0028, 0.0028])
    ops.mass(1403, *[0.0016, 0.0016, 0.0016])
    ops.mass(1404, *[0.0025, 0.0025, 0.0025])
    ops.mass(1405, *[0.0016, 0.0016, 0.0016])
    ops.mass(1406, *[0.0029, 0.0029, 0.0029])
    ops.mass(1407, *[0.0011, 0.0011, 0.0011])
    ops.mass(1408, *[0.0023, 0.0023, 0.0023])
    ops.mass(1409, *[0.002, 0.002, 0.002])
    ops.mass(1410, *[0.003, 0.003, 0.003])
    ops.mass(1411, *[0.0023, 0.0023, 0.0023])
    ops.mass(1412, *[0.001, 0.001, 0.001])
    ops.mass(1413, *[0.0017, 0.0017, 0.0017])
    ops.mass(1414, *[0.0021, 0.0021, 0.0021])
    ops.mass(1415, *[0.0025, 0.0025, 0.0025])
    ops.mass(1416, *[0.0015, 0.0015, 0.0015])
    ops.mass(1417, *[0.0019, 0.0019, 0.0019])
    ops.mass(1418, *[0.0025, 0.0025, 0.0025])
    ops.mass(1419, *[0.0025, 0.0025, 0.0025])
    ops.mass(1420, *[0.0031, 0.0031, 0.0031])
    ops.mass(1421, *[0.0021, 0.0021, 0.0021])
    ops.mass(1422, *[0.0021, 0.0021, 0.0021])
    ops.mass(1423, *[0.0024, 0.0024, 0.0024])
    ops.mass(1424, *[0.0016, 0.0016, 0.0016])
    ops.mass(1425, *[0.0027, 0.0027, 0.0027])
    ops.mass(1426, *[0.0019, 0.0019, 0.0019])
    ops.mass(1427, *[0.0017, 0.0017, 0.0017])
    ops.mass(1428, *[0.0019, 0.0019, 0.0019])
    ops.mass(1429, *[0.0032, 0.0032, 0.0032])
    ops.mass(1430, *[0.0034, 0.0034, 0.0034])
    ops.mass(1431, *[0.0026, 0.0026, 0.0026])
    ops.mass(1432, *[0.0023, 0.0023, 0.0023])
    ops.mass(1433, *[0.0024, 0.0024, 0.0024])
    ops.mass(1434, *[0.0033, 0.0033, 0.0033])
    ops.mass(1435, *[0.0027, 0.0027, 0.0027])
    ops.mass(1436, *[0.002, 0.002, 0.002])
    ops.mass(1437, *[0.0024, 0.0024, 0.0024])
    ops.mass(1438, *[0.0031, 0.0031, 0.0031])
    ops.mass(1439, *[0.0021, 0.0021, 0.0021])
    ops.mass(1440, *[0.0024, 0.0024, 0.0024])
    ops.mass(1441, *[0.0023, 0.0023, 0.0023])
    ops.mass(1442, *[0.0023, 0.0023, 0.0023])
    ops.mass(1443, *[0.0024, 0.0024, 0.0024])
    ops.mass(1444, *[0.0019, 0.0019, 0.0019])
    ops.mass(1445, *[0.0022, 0.0022, 0.0022])
    ops.mass(1446, *[0.0021, 0.0021, 0.0021])
    ops.mass(1447, *[0.0037, 0.0037, 0.0037])
    ops.mass(1448, *[0.0039, 0.0039, 0.0039])
    ops.mass(1449, *[0.0011, 0.0011, 0.0011])
    ops.mass(1450, *[0.0019, 0.0019, 0.0019])
    ops.mass(1451, *[0.0027, 0.0027, 0.0027])
    ops.mass(1452, *[0.0026, 0.0026, 0.0026])
    ops.mass(1453, *[0.0031, 0.0031, 0.0031])
    ops.mass(1454, *[0.0025, 0.0025, 0.0025])
    ops.mass(1455, *[0.0024, 0.0024, 0.0024])
    ops.mass(1456, *[0.0026, 0.0026, 0.0026])
    ops.mass(1457, *[0.0021, 0.0021, 0.0021])
    ops.mass(1458, *[0.0016, 0.0016, 0.0016])
    ops.mass(1459, *[0.0023, 0.0023, 0.0023])
    ops.mass(1460, *[0.0022, 0.0022, 0.0022])
    ops.mass(1461, *[0.002, 0.002, 0.002])
    ops.mass(1462, *[0.0054, 0.0054, 0.0054])
    ops.mass(1463, *[0.0013, 0.0013, 0.0013])
    ops.mass(1464, *[0.0021, 0.0021, 0.0021])
    ops.mass(1465, *[0.0077, 0.0077, 0.0077])
    ops.mass(1466, *[0.0033, 0.0033, 0.0033])
    ops.mass(1467, *[0.0019, 0.0019, 0.0019])
    ops.mass(1468, *[0.0019, 0.0019, 0.0019])
    ops.mass(1469, *[0.0029, 0.0029, 0.0029])
    ops.mass(1470, *[0.0025, 0.0025, 0.0025])
    ops.mass(1471, *[0.0022, 0.0022, 0.0022])
    ops.mass(1472, *[0.002, 0.002, 0.002])
    ops.mass(1473, *[0.0016, 0.0016, 0.0016])
    ops.mass(1474, *[0.0012, 0.0012, 0.0012])
    ops.mass(1475, *[0.004, 0.004, 0.004])
    ops.mass(1476, *[0.002, 0.002, 0.002])
    ops.mass(1477, *[0.0026, 0.0026, 0.0026])
    ops.mass(1478, *[0.0015, 0.0015, 0.0015])
    ops.mass(1479, *[0.0018, 0.0018, 0.0018])
    ops.mass(1480, *[0.0037, 0.0037, 0.0037])
    ops.mass(1481, *[0.0027, 0.0027, 0.0027])
    ops.mass(1482, *[0.0007, 0.0007, 0.0007])
    ops.mass(1483, *[0.0023, 0.0023, 0.0023])
    ops.mass(1484, *[0.0016, 0.0016, 0.0016])
    ops.mass(1485, *[0.0028, 0.0028, 0.0028])
    ops.mass(1486, *[0.002, 0.002, 0.002])
    ops.mass(1487, *[0.003, 0.003, 0.003])
    ops.mass(1488, *[0.0018, 0.0018, 0.0018])
    ops.mass(1489, *[0.0026, 0.0026, 0.0026])
    ops.mass(1490, *[0.0026, 0.0026, 0.0026])
    ops.mass(1491, *[0.0014, 0.0014, 0.0014])
    ops.mass(1492, *[0.0027, 0.0027, 0.0027])
    ops.mass(1493, *[0.0061, 0.0061, 0.0061])
    ops.mass(1494, *[0.0063, 0.0063, 0.0063])
    ops.mass(1495, *[0.0023, 0.0023, 0.0023])
    ops.mass(1496, *[0.0023, 0.0023, 0.0023])
    ops.mass(1497, *[0.0051, 0.0051, 0.0051])
    ops.mass(1498, *[0.0012, 0.0012, 0.0012])
    ops.mass(1499, *[0.0025, 0.0025, 0.0025])
    ops.mass(1500, *[0.0036, 0.0036, 0.0036])
    ops.mass(1501, *[0.0022, 0.0022, 0.0022])
    ops.mass(1502, *[0.0023, 0.0023, 0.0023])
    ops.mass(1503, *[0.0032, 0.0032, 0.0032])
    ops.mass(1504, *[0.0018, 0.0018, 0.0018])
    ops.mass(1505, *[0.0022, 0.0022, 0.0022])
    ops.mass(1506, *[0.0023, 0.0023, 0.0023])
    ops.mass(1507, *[0.0045, 0.0045, 0.0045])
    ops.mass(1508, *[0.0016, 0.0016, 0.0016])
    ops.mass(1509, *[0.0037, 0.0037, 0.0037])
    ops.mass(1510, *[0.0034, 0.0034, 0.0034])
    ops.mass(1511, *[0.0026, 0.0026, 0.0026])
    ops.mass(1512, *[0.0028, 0.0028, 0.0028])
    ops.mass(1513, *[0.0022, 0.0022, 0.0022])
    ops.mass(1514, *[0.0022, 0.0022, 0.0022])
    ops.mass(1515, *[0.0023, 0.0023, 0.0023])
    ops.mass(1516, *[0.0086, 0.0086, 0.0086])
    ops.mass(1517, *[0.0063, 0.0063, 0.0063])
    ops.mass(1518, *[0.0074, 0.0074, 0.0074])
    ops.mass(1519, *[0.0038, 0.0038, 0.0038])
    ops.mass(1520, *[0.0043, 0.0043, 0.0043])
    ops.mass(1521, *[0.0042, 0.0042, 0.0042])
    ops.mass(1522, *[0.0034, 0.0034, 0.0034])
    ops.mass(1523, *[0.0034, 0.0034, 0.0034])
    ops.mass(1524, *[0.0033, 0.0033, 0.0033])
    ops.mass(1525, *[0.0046, 0.0046, 0.0046])
    ops.mass(1526, *[0.005, 0.005, 0.005])
    ops.mass(1527, *[0.0018, 0.0018, 0.0018])
    ops.mass(1528, *[0.0028, 0.0028, 0.0028])
    ops.mass(1529, *[0.0007, 0.0007, 0.0007])
    ops.mass(1530, *[0.0017, 0.0017, 0.0017])
    ops.mass(1531, *[0.0013, 0.0013, 0.0013])
    ops.mass(1532, *[0.0012, 0.0012, 0.0012])
    ops.mass(1533, *[0.0012, 0.0012, 0.0012])
    ops.mass(1534, *[0.0016, 0.0016, 0.0016])
    ops.mass(1535, *[0.0013, 0.0013, 0.0013])
    ops.mass(1536, *[0.0017, 0.0017, 0.0017])
    ops.mass(1537, *[0.0017, 0.0017, 0.0017])
    ops.mass(1538, *[0.0016, 0.0016, 0.0016])
    ops.mass(1539, *[0.0009, 0.0009, 0.0009])
    ops.mass(1540, *[0.0013, 0.0013, 0.0013])
    ops.mass(1541, *[0.001, 0.001, 0.001])
    ops.mass(1542, *[0.0017, 0.0017, 0.0017])
    ops.mass(1543, *[0.0022, 0.0022, 0.0022])
    ops.mass(1544, *[0.0021, 0.0021, 0.0021])
    ops.mass(1545, *[0.0014, 0.0014, 0.0014])
    ops.mass(1546, *[0.0019, 0.0019, 0.0019])
    ops.mass(1547, *[0.0015, 0.0015, 0.0015])
    ops.mass(1548, *[0.0024, 0.0024, 0.0024])
    ops.mass(1549, *[0.0017, 0.0017, 0.0017])
    ops.mass(1550, *[0.0017, 0.0017, 0.0017])
    ops.mass(1551, *[0.0016, 0.0016, 0.0016])
    ops.mass(1552, *[0.0016, 0.0016, 0.0016])
    ops.mass(1553, *[0.0017, 0.0017, 0.0017])
    ops.mass(1554, *[0.0014, 0.0014, 0.0014])
    ops.mass(1555, *[0.0016, 0.0016, 0.0016])
    ops.mass(1556, *[0.0018, 0.0018, 0.0018])
    ops.mass(1557, *[0.0017, 0.0017, 0.0017])
    ops.mass(1558, *[0.0012, 0.0012, 0.0012])
    ops.mass(1559, *[0.0011, 0.0011, 0.0011])
    ops.mass(1560, *[0.0024, 0.0024, 0.0024])
    ops.mass(1561, *[0.0014, 0.0014, 0.0014])
    ops.mass(1562, *[0.0012, 0.0012, 0.0012])
    ops.mass(1563, *[0.0015, 0.0015, 0.0015])
    ops.mass(1564, *[0.002, 0.002, 0.002])
    ops.mass(1565, *[0.0017, 0.0017, 0.0017])
    ops.mass(1566, *[0.0021, 0.0021, 0.0021])
    ops.mass(1567, *[0.0015, 0.0015, 0.0015])
    ops.mass(1568, *[0.0016, 0.0016, 0.0016])
    ops.mass(1569, *[0.002, 0.002, 0.002])
    ops.mass(1570, *[0.0022, 0.0022, 0.0022])
    ops.mass(1571, *[0.0019, 0.0019, 0.0019])
    ops.mass(1572, *[0.0023, 0.0023, 0.0023])
    ops.mass(1573, *[0.0029, 0.0029, 0.0029])
    ops.mass(1574, *[0.0022, 0.0022, 0.0022])
    ops.mass(1575, *[0.0016, 0.0016, 0.0016])
    ops.mass(1576, *[0.0013, 0.0013, 0.0013])
    ops.mass(1577, *[0.0012, 0.0012, 0.0012])
    ops.mass(1578, *[0.0021, 0.0021, 0.0021])
    ops.mass(1579, *[0.0021, 0.0021, 0.0021])
    ops.mass(1580, *[0.0019, 0.0019, 0.0019])
    ops.mass(1581, *[0.0022, 0.0022, 0.0022])
    ops.mass(1582, *[0.002, 0.002, 0.002])
    ops.mass(1583, *[0.0019, 0.0019, 0.0019])
    ops.mass(1584, *[0.0021, 0.0021, 0.0021])
    ops.mass(1585, *[0.002, 0.002, 0.002])
    ops.mass(1586, *[0.0026, 0.0026, 0.0026])
    ops.mass(1587, *[0.0017, 0.0017, 0.0017])
    ops.mass(1588, *[0.0017, 0.0017, 0.0017])
    ops.mass(1589, *[0.0033, 0.0033, 0.0033])
    ops.mass(1590, *[0.0023, 0.0023, 0.0023])
    ops.mass(1591, *[0.0019, 0.0019, 0.0019])
    ops.mass(1592, *[0.0022, 0.0022, 0.0022])
    ops.mass(1593, *[0.0019, 0.0019, 0.0019])
    ops.mass(1594, *[0.0018, 0.0018, 0.0018])
    ops.mass(1595, *[0.0017, 0.0017, 0.0017])
    ops.mass(1596, *[0.0021, 0.0021, 0.0021])
    ops.mass(1597, *[0.0016, 0.0016, 0.0016])
    ops.mass(1598, *[0.0018, 0.0018, 0.0018])
    ops.mass(1599, *[0.0031, 0.0031, 0.0031])
    ops.mass(1600, *[0.0036, 0.0036, 0.0036])
    ops.mass(1601, *[0.0012, 0.0012, 0.0012])
    ops.mass(1602, *[0.001, 0.001, 0.001])
    ops.mass(1603, *[0.0022, 0.0022, 0.0022])
    ops.mass(1604, *[0.0032, 0.0032, 0.0032])
    ops.mass(1605, *[0.0012, 0.0012, 0.0012])
    ops.mass(1606, *[0.0013, 0.0013, 0.0013])
    ops.mass(1607, *[0.0026, 0.0026, 0.0026])
    ops.mass(1608, *[0.0022, 0.0022, 0.0022])
    ops.mass(1609, *[0.0016, 0.0016, 0.0016])
    ops.mass(1610, *[0.0027, 0.0027, 0.0027])
    ops.mass(1611, *[0.002, 0.002, 0.002])
    ops.mass(1612, *[0.002, 0.002, 0.002])
    ops.mass(1613, *[0.0048, 0.0048, 0.0048])
    ops.mass(1614, *[0.007, 0.007, 0.007])
    ops.mass(1615, *[0.0031, 0.0031, 0.0031])
    ops.mass(1616, *[0.0066, 0.0066, 0.0066])
    ops.mass(1617, *[0.0014, 0.0014, 0.0014])
    ops.mass(1618, *[0.0017, 0.0017, 0.0017])
    ops.mass(1619, *[0.0017, 0.0017, 0.0017])
    ops.mass(1620, *[0.0016, 0.0016, 0.0016])
    ops.mass(1621, *[0.002, 0.002, 0.002])
    ops.mass(1622, *[0.0021, 0.0021, 0.0021])
    ops.mass(1623, *[0.0012, 0.0012, 0.0012])
    ops.mass(1624, *[0.0014, 0.0014, 0.0014])
    ops.mass(1625, *[0.0024, 0.0024, 0.0024])
    ops.mass(1626, *[0.0013, 0.0013, 0.0013])
    ops.mass(1627, *[0.0026, 0.0026, 0.0026])
    ops.mass(1628, *[0.0009, 0.0009, 0.0009])
    ops.mass(1629, *[0.0016, 0.0016, 0.0016])
    ops.mass(1630, *[0.0017, 0.0017, 0.0017])
    ops.mass(1631, *[0.002, 0.002, 0.002])
    ops.mass(1632, *[0.0013, 0.0013, 0.0013])
    ops.mass(1633, *[0.0015, 0.0015, 0.0015])
    ops.mass(1634, *[0.0015, 0.0015, 0.0015])
    ops.mass(1635, *[0.0027, 0.0027, 0.0027])
    ops.mass(1636, *[0.0017, 0.0017, 0.0017])
    ops.mass(1637, *[0.0024, 0.0024, 0.0024])
    ops.mass(1638, *[0.0018, 0.0018, 0.0018])
    ops.mass(1639, *[0.0019, 0.0019, 0.0019])
    ops.mass(1640, *[0.002, 0.002, 0.002])
    ops.mass(1641, *[0.0018, 0.0018, 0.0018])
    ops.mass(1642, *[0.0022, 0.0022, 0.0022])
    ops.mass(1643, *[0.0028, 0.0028, 0.0028])
    ops.mass(1644, *[0.0032, 0.0032, 0.0032])
    ops.mass(1645, *[0.0027, 0.0027, 0.0027])
    ops.mass(1646, *[0.0035, 0.0035, 0.0035])
    ops.mass(1647, *[0.0066, 0.0066, 0.0066])
    ops.mass(1648, *[0.0037, 0.0037, 0.0037])
    ops.mass(1649, *[0.0057, 0.0057, 0.0057])
    ops.mass(1650, *[0.0047, 0.0047, 0.0047])
    ops.mass(1651, *[0.0012, 0.0012, 0.0012])
    ops.mass(1652, *[0.0023, 0.0023, 0.0023])
    ops.mass(1653, *[0.0024, 0.0024, 0.0024])
    ops.mass(1654, *[0.002, 0.002, 0.002])
    ops.mass(1655, *[0.0027, 0.0027, 0.0027])
    ops.mass(1656, *[0.0019, 0.0019, 0.0019])
    ops.mass(1657, *[0.0006, 0.0006, 0.0006])
    ops.mass(1658, *[0.0012, 0.0012, 0.0012])
    ops.mass(1659, *[0.0016, 0.0016, 0.0016])
    ops.mass(1660, *[0.0019, 0.0019, 0.0019])
    ops.mass(1661, *[0.0018, 0.0018, 0.0018])
    ops.mass(1662, *[0.0007, 0.0007, 0.0007])
    ops.mass(1663, *[0.0008, 0.0008, 0.0008])
    ops.mass(1664, *[0.0007, 0.0007, 0.0007])
    ops.mass(1665, *[0.0007, 0.0007, 0.0007])
    ops.mass(1666, *[0.0007, 0.0007, 0.0007])
    ops.mass(1667, *[0.0017, 0.0017, 0.0017])
    ops.mass(1668, *[0.0017, 0.0017, 0.0017])
    ops.mass(1669, *[0.002, 0.002, 0.002])
    ops.mass(1670, *[0.0012, 0.0012, 0.0012])
    ops.mass(1671, *[0.0015, 0.0015, 0.0015])
    ops.mass(1672, *[0.0012, 0.0012, 0.0012])
    ops.mass(1673, *[0.001, 0.001, 0.001])
    ops.mass(1674, *[0.0015, 0.0015, 0.0015])
    ops.mass(1675, *[0.0013, 0.0013, 0.0013])
    ops.mass(1676, *[0.0007, 0.0007, 0.0007])
    ops.mass(1677, *[0.002, 0.002, 0.002])
    ops.mass(1678, *[0.0006, 0.0006, 0.0006])
    ops.mass(1679, *[0.0011, 0.0011, 0.0011])
    ops.mass(1680, *[0.0004, 0.0004, 0.0004])
    ops.mass(1681, *[0.0031, 0.0031, 0.0031])
    ops.mass(1682, *[0.0011, 0.0011, 0.0011])
    ops.mass(1683, *[0.0015, 0.0015, 0.0015])
    ops.mass(1684, *[0.0007, 0.0007, 0.0007])
    ops.mass(1685, *[0.0018, 0.0018, 0.0018])
    ops.mass(1686, *[0.0004, 0.0004, 0.0004])
    ops.mass(1687, *[0.0007, 0.0007, 0.0007])
    ops.mass(1688, *[0.002, 0.002, 0.002])
    ops.mass(1689, *[0.0021, 0.0021, 0.0021])
    ops.mass(1690, *[0.0013, 0.0013, 0.0013])
    ops.mass(1691, *[0.001, 0.001, 0.001])
    ops.mass(1692, *[0.0007, 0.0007, 0.0007])
    ops.mass(1693, *[0.0016, 0.0016, 0.0016])
    ops.mass(1694, *[0.001, 0.001, 0.001])
    ops.mass(1695, *[0.0011, 0.0011, 0.0011])
    ops.mass(1696, *[0.0007, 0.0007, 0.0007])
    ops.mass(1697, *[0.0016, 0.0016, 0.0016])
    ops.mass(1698, *[0.0012, 0.0012, 0.0012])
    ops.mass(1699, *[0.0013, 0.0013, 0.0013])
    ops.mass(1700, *[0.0012, 0.0012, 0.0012])
    ops.mass(1701, *[0.0015, 0.0015, 0.0015])
    ops.mass(1702, *[0.0016, 0.0016, 0.0016])
    ops.mass(1703, *[0.0015, 0.0015, 0.0015])
    ops.mass(1704, *[0.001, 0.001, 0.001])
    ops.mass(1705, *[0.0013, 0.0013, 0.0013])
    ops.mass(1706, *[0.0014, 0.0014, 0.0014])
    ops.mass(1707, *[0.0013, 0.0013, 0.0013])
    ops.mass(1708, *[0.0008, 0.0008, 0.0008])
    ops.mass(1709, *[0.0027, 0.0027, 0.0027])
    ops.mass(1710, *[0.001, 0.001, 0.001])
    ops.mass(1711, *[0.0015, 0.0015, 0.0015])
    ops.mass(1712, *[0.0015, 0.0015, 0.0015])
    ops.mass(1713, *[0.0009, 0.0009, 0.0009])
    ops.mass(1714, *[0.0019, 0.0019, 0.0019])
    ops.mass(1715, *[0.0008, 0.0008, 0.0008])
    ops.mass(1716, *[0.0022, 0.0022, 0.0022])
    ops.mass(1717, *[0.0013, 0.0013, 0.0013])
    ops.mass(1718, *[0.0026, 0.0026, 0.0026])
    ops.mass(1719, *[0.0025, 0.0025, 0.0025])
    ops.mass(1720, *[0.0011, 0.0011, 0.0011])
    ops.mass(1721, *[0.0016, 0.0016, 0.0016])
    ops.mass(1722, *[0.0025, 0.0025, 0.0025])
    ops.mass(1723, *[0.0016, 0.0016, 0.0016])
    ops.mass(1724, *[0.0024, 0.0024, 0.0024])
    ops.mass(1725, *[0.0018, 0.0018, 0.0018])
    ops.mass(1726, *[0.0015, 0.0015, 0.0015])
    ops.mass(1727, *[0.002, 0.002, 0.002])
    ops.mass(1728, *[0.0019, 0.0019, 0.0019])
    ops.mass(1729, *[0.0011, 0.0011, 0.0011])
    ops.mass(1730, *[0.0014, 0.0014, 0.0014])
    ops.mass(1731, *[0.002, 0.002, 0.002])
    ops.mass(1732, *[0.0006, 0.0006, 0.0006])
    ops.mass(1733, *[0.0017, 0.0017, 0.0017])
    ops.mass(1734, *[0.0014, 0.0014, 0.0014])
    ops.mass(1735, *[0.0004, 0.0004, 0.0004])
    ops.mass(1736, *[0.0001, 0.0001, 0.0001])
    ops.mass(1737, *[0.0003, 0.0003, 0.0003])
    ops.mass(1738, *[0.0006, 0.0006, 0.0006])
    ops.mass(1739, *[0.0005, 0.0005, 0.0005])
    ops.mass(1740, *[0.0007, 0.0007, 0.0007])
    ops.mass(1741, *[0.0006, 0.0006, 0.0006])
    ops.mass(1742, *[0.0003, 0.0003, 0.0003])
    ops.mass(1743, *[0.0007, 0.0007, 0.0007])
    ops.mass(1744, *[0.0006, 0.0006, 0.0006])
    ops.mass(1745, *[0.0004, 0.0004, 0.0004])
    ops.mass(1746, *[0.0016, 0.0016, 0.0016])
    ops.mass(1747, *[0.0054, 0.0054, 0.0054])
    ops.mass(1748, *[0.0016, 0.0016, 0.0016])
    ops.mass(1749, *[0.0008, 0.0008, 0.0008])
    ops.mass(1750, *[0.0033, 0.0033, 0.0033])
    ops.mass(1751, *[0.0007, 0.0007, 0.0007])
    ops.mass(1752, *[0.0014, 0.0014, 0.0014])
    ops.mass(1753, *[0.0012, 0.0012, 0.0012])
    ops.mass(1754, *[0.002, 0.002, 0.002])
    ops.mass(1755, *[0.0014, 0.0014, 0.0014])
    ops.mass(1756, *[0.0008, 0.0008, 0.0008])
    ops.mass(1757, *[0.0007, 0.0007, 0.0007])
    ops.mass(1758, *[0.0013, 0.0013, 0.0013])
    ops.mass(1759, *[0.0012, 0.0012, 0.0012])
    ops.mass(1760, *[0.0021, 0.0021, 0.0021])
    ops.mass(1761, *[0.0009, 0.0009, 0.0009])
    ops.mass(1762, *[0.0025, 0.0025, 0.0025])
    ops.mass(1763, *[0.0028, 0.0028, 0.0028])
    ops.mass(1764, *[0.0012, 0.0012, 0.0012])
    ops.mass(1765, *[0.0012, 0.0012, 0.0012])
    ops.mass(1766, *[0.0004, 0.0004, 0.0004])
    ops.mass(1767, *[0.0007, 0.0007, 0.0007])
    ops.mass(1768, *[0.0019, 0.0019, 0.0019])
    ops.mass(1769, *[0.001, 0.001, 0.001])
    ops.mass(1770, *[0.001, 0.001, 0.001])
    ops.mass(1771, *[0.0016, 0.0016, 0.0016])
    ops.mass(1772, *[0.0012, 0.0012, 0.0012])
    ops.mass(1773, *[0.0005, 0.0005, 0.0005])
    ops.mass(1774, *[0.0006, 0.0006, 0.0006])
    ops.mass(1775, *[0.0007, 0.0007, 0.0007])
    ops.mass(1776, *[0.002, 0.002, 0.002])
    ops.mass(1777, *[0.0006, 0.0006, 0.0006])
    ops.mass(1778, *[0.0009, 0.0009, 0.0009])
    ops.mass(1779, *[0.002, 0.002, 0.002])
    ops.mass(1780, *[0.0021, 0.0021, 0.0021])
    ops.mass(1781, *[0.0024, 0.0024, 0.0024])
    ops.mass(1782, *[0.0006, 0.0006, 0.0006])
    ops.mass(1783, *[0.0007, 0.0007, 0.0007])
    ops.mass(1784, *[0.0063, 0.0063, 0.0063])
    ops.mass(1785, *[0.0028, 0.0028, 0.0028])
    ops.mass(1786, *[0.0018, 0.0018, 0.0018])
    ops.mass(1787, *[0.0031, 0.0031, 0.0031])
    ops.mass(1788, *[0.0003, 0.0003, 0.0003])
    ops.mass(1789, *[0.0004, 0.0004, 0.0004])
    ops.mass(1790, *[0.001, 0.001, 0.001])
    ops.mass(1791, *[0.0003, 0.0003, 0.0003])
    ops.mass(1792, *[0.0007, 0.0007, 0.0007])
    ops.mass(1793, *[0.0005, 0.0005, 0.0005])
    ops.mass(1794, *[0.0005, 0.0005, 0.0005])
    ops.mass(1795, *[0.0004, 0.0004, 0.0004])
    ops.mass(1796, *[0.002, 0.002, 0.002])
    ops.mass(1797, *[0.0019, 0.0019, 0.0019])
    ops.mass(1798, *[0.0008, 0.0008, 0.0008])
    ops.mass(1799, *[0.0018, 0.0018, 0.0018])
    ops.mass(1800, *[0.0023, 0.0023, 0.0023])
    ops.mass(1801, *[0.0006, 0.0006, 0.0006])
    ops.mass(1802, *[0.0007, 0.0007, 0.0007])
    ops.mass(1803, *[0.0016, 0.0016, 0.0016])
    ops.mass(1804, *[0.0006, 0.0006, 0.0006])
    ops.mass(1805, *[0.0008, 0.0008, 0.0008])
    ops.mass(1806, *[0.0017, 0.0017, 0.0017])
    ops.mass(1807, *[0.0011, 0.0011, 0.0011])
    ops.mass(1808, *[0.0017, 0.0017, 0.0017])
    ops.mass(1809, *[0.0031, 0.0031, 0.0031])
    ops.mass(1810, *[0.0021, 0.0021, 0.0021])
    ops.mass(1811, *[0.0029, 0.0029, 0.0029])
    ops.mass(1812, *[0.0004, 0.0004, 0.0004])
    ops.mass(1813, *[0.0003, 0.0003, 0.0003])
    ops.mass(1814, *[0.0018, 0.0018, 0.0018])
    ops.mass(1815, *[0.0004, 0.0004, 0.0004])
    ops.mass(1816, *[0.0006, 0.0006, 0.0006])
    ops.mass(1817, *[0.0007, 0.0007, 0.0007])
    ops.mass(1818, *[0.0015, 0.0015, 0.0015])
    ops.mass(1819, *[0.0016, 0.0016, 0.0016])
    ops.mass(1820, *[0.0007, 0.0007, 0.0007])
    ops.mass(1821, *[0.0019, 0.0019, 0.0019])
    ops.mass(1822, *[0.0002, 0.0002, 0.0002])
    ops.mass(1823, *[0.0012, 0.0012, 0.0012])
    ops.mass(1824, *[0.0009, 0.0009, 0.0009])
    ops.mass(1825, *[0.0025, 0.0025, 0.0025])
    ops.mass(1826, *[0.001, 0.001, 0.001])
    ops.mass(1827, *[0.0009, 0.0009, 0.0009])
    ops.mass(1828, *[0.0007, 0.0007, 0.0007])
    ops.mass(1829, *[0.0016, 0.0016, 0.0016])
    ops.mass(1830, *[0.0017, 0.0017, 0.0017])
    ops.mass(1831, *[0.0006, 0.0006, 0.0006])
    ops.mass(1832, *[0.0023, 0.0023, 0.0023])
    ops.mass(1833, *[0.001, 0.001, 0.001])
    ops.mass(1834, *[0.0018, 0.0018, 0.0018])
    ops.mass(1835, *[0.0017, 0.0017, 0.0017])
    ops.mass(1836, *[0.0022, 0.0022, 0.0022])
    ops.mass(1837, *[0.0004, 0.0004, 0.0004])
    ops.mass(1838, *[0.0021, 0.0021, 0.0021])
    ops.mass(1839, *[0.0005, 0.0005, 0.0005])
    ops.mass(1840, *[0.001, 0.001, 0.001])
    ops.mass(1841, *[0.0025, 0.0025, 0.0025])
    ops.mass(1842, *[0.0036, 0.0036, 0.0036])
    ops.mass(1843, *[0.0019, 0.0019, 0.0019])
    ops.mass(1844, *[0.0086, 0.0086, 0.0086])
    ops.mass(1845, *[0.0007, 0.0007, 0.0007])
    ops.mass(1846, *[0.0008, 0.0008, 0.0008])
    ops.mass(1847, *[0.0013, 0.0013, 0.0013])
    ops.mass(1848, *[0.0017, 0.0017, 0.0017])
    ops.mass(1849, *[0.0014, 0.0014, 0.0014])
    ops.mass(1850, *[0.0002, 0.0002, 0.0002])
    ops.mass(1851, *[0.0032, 0.0032, 0.0032])
    ops.mass(1852, *[0.0032, 0.0032, 0.0032])
    ops.mass(1853, *[0.0026, 0.0026, 0.0026])
    ops.mass(1854, *[0.0024, 0.0024, 0.0024])
    ops.mass(1855, *[0.0015, 0.0015, 0.0015])
    ops.mass(1856, *[0.0015, 0.0015, 0.0015])
    ops.mass(1857, *[0.0013, 0.0013, 0.0013])
    ops.mass(1858, *[0.0013, 0.0013, 0.0013])
    ops.mass(1859, *[0.0037, 0.0037, 0.0037])
    ops.mass(1860, *[0.0034, 0.0034, 0.0034])
    ops.mass(1861, *[0.0031, 0.0031, 0.0031])
    ops.mass(1862, *[0.0016, 0.0016, 0.0016])
    ops.mass(1863, *[0.0007, 0.0007, 0.0007])
    ops.mass(1864, *[0.0006, 0.0006, 0.0006])
    ops.mass(1865, *[0.0011, 0.0011, 0.0011])
    ops.mass(1866, *[0.0026, 0.0026, 0.0026])
    ops.mass(1867, *[0.0016, 0.0016, 0.0016])
    ops.mass(1868, *[0.0023, 0.0023, 0.0023])
    ops.mass(1869, *[0.0025, 0.0025, 0.0025])
    ops.mass(1870, *[0.0023, 0.0023, 0.0023])
    ops.mass(1871, *[0.0015, 0.0015, 0.0015])
    ops.mass(1872, *[0.0016, 0.0016, 0.0016])
    ops.mass(1873, *[0.0005, 0.0005, 0.0005])
    ops.mass(1874, *[0.0002, 0.0002, 0.0002])
    ops.mass(1875, *[0.0007, 0.0007, 0.0007])
    ops.mass(1876, *[0.0006, 0.0006, 0.0006])
    ops.mass(1877, *[0.0003, 0.0003, 0.0003])
    ops.mass(1878, *[0.0011, 0.0011, 0.0011])
    ops.mass(1879, *[0.0021, 0.0021, 0.0021])
    ops.mass(1880, *[0.0017, 0.0017, 0.0017])
    ops.mass(1881, *[0.0017, 0.0017, 0.0017])
    ops.mass(1882, *[0.0019, 0.0019, 0.0019])
    ops.mass(1883, *[0.0019, 0.0019, 0.0019])
    ops.mass(1884, *[0.0023, 0.0023, 0.0023])
    ops.mass(1885, *[0.0015, 0.0015, 0.0015])
    ops.mass(1886, *[0.0023, 0.0023, 0.0023])
    ops.mass(1887, *[0.0013, 0.0013, 0.0013])
    ops.mass(1888, *[0.0009, 0.0009, 0.0009])
    ops.mass(1889, *[0.001, 0.001, 0.001])
    ops.mass(1890, *[0.0014, 0.0014, 0.0014])
    ops.mass(1891, *[0.0038, 0.0038, 0.0038])
    ops.mass(1892, *[0.0017, 0.0017, 0.0017])
    ops.mass(1893, *[0.0004, 0.0004, 0.0004])
    ops.mass(1894, *[0.0009, 0.0009, 0.0009])
    ops.mass(1895, *[0.0015, 0.0015, 0.0015])
    ops.mass(1896, *[0.0012, 0.0012, 0.0012])
    ops.mass(1897, *[0.001, 0.001, 0.001])
    ops.mass(1898, *[0.0037, 0.0037, 0.0037])
    ops.mass(1899, *[0.0023, 0.0023, 0.0023])
    ops.mass(1900, *[0.0018, 0.0018, 0.0018])
    ops.mass(1901, *[0.0016, 0.0016, 0.0016])
    ops.mass(1902, *[0.0014, 0.0014, 0.0014])
    ops.mass(1903, *[0.0031, 0.0031, 0.0031])
    ops.mass(1904, *[0.0014, 0.0014, 0.0014])
    ops.mass(1905, *[0.0014, 0.0014, 0.0014])
    ops.mass(1906, *[0.0007, 0.0007, 0.0007])
    ops.mass(1907, *[0.0005, 0.0005, 0.0005])
    ops.mass(1908, *[0.001, 0.001, 0.001])
    ops.mass(1909, *[0.0004, 0.0004, 0.0004])
    ops.mass(1910, *[0.0003, 0.0003, 0.0003])
    ops.mass(1911, *[0.0004, 0.0004, 0.0004])
    ops.mass(1912, *[0.0008, 0.0008, 0.0008])
    ops.mass(1913, *[0.0046, 0.0046, 0.0046])
    ops.mass(1914, *[0.001, 0.001, 0.001])
    ops.mass(1915, *[0.0013, 0.0013, 0.0013])
    ops.mass(1916, *[0.0013, 0.0013, 0.0013])
    ops.mass(1917, *[0.0018, 0.0018, 0.0018])
    ops.mass(1918, *[0.0009, 0.0009, 0.0009])
    ops.mass(1919, *[0.0005, 0.0005, 0.0005])
    ops.mass(1920, *[0.0016, 0.0016, 0.0016])
    ops.mass(1921, *[0.0008, 0.0008, 0.0008])
    ops.mass(1922, *[0.0006, 0.0006, 0.0006])
    ops.mass(1923, *[0.0027, 0.0027, 0.0027])
    ops.mass(1924, *[0.0018, 0.0018, 0.0018])
    ops.mass(1925, *[0.0016, 0.0016, 0.0016])
    ops.mass(1926, *[0.0032, 0.0032, 0.0032])
    ops.mass(1927, *[0.0021, 0.0021, 0.0021])
    ops.mass(1928, *[0.0031, 0.0031, 0.0031])
    ops.mass(1929, *[0.0053, 0.0053, 0.0053])
    ops.mass(1930, *[0.0027, 0.0027, 0.0027])
    ops.mass(1931, *[0.0024, 0.0024, 0.0024])
    ops.mass(1932, *[0.0017, 0.0017, 0.0017])
    ops.mass(1933, *[0.0017, 0.0017, 0.0017])
    ops.mass(1934, *[0.0014, 0.0014, 0.0014])
    ops.mass(1935, *[0.0026, 0.0026, 0.0026])
    ops.mass(1936, *[0.0017, 0.0017, 0.0017])
    ops.mass(1937, *[0.0022, 0.0022, 0.0022])
    ops.mass(1938, *[0.0006, 0.0006, 0.0006])
    ops.mass(1939, *[0.0008, 0.0008, 0.0008])
    ops.mass(1940, *[0.0007, 0.0007, 0.0007])
    ops.mass(1941, *[0.0029, 0.0029, 0.0029])
    ops.mass(1942, *[0.0021, 0.0021, 0.0021])
    ops.mass(1943, *[0.0023, 0.0023, 0.0023])
    ops.mass(1944, *[0.0007, 0.0007, 0.0007])
    ops.mass(1945, *[0.0008, 0.0008, 0.0008])
    ops.mass(1946, *[0.0009, 0.0009, 0.0009])
    ops.mass(1947, *[0.0005, 0.0005, 0.0005])
    ops.mass(1948, *[0.0013, 0.0013, 0.0013])
    ops.mass(1949, *[0.0037, 0.0037, 0.0037])
    ops.mass(1950, *[0.0015, 0.0015, 0.0015])
    ops.mass(1951, *[0.0008, 0.0008, 0.0008])
    ops.mass(1952, *[0.0027, 0.0027, 0.0027])
    ops.mass(1953, *[0.0023, 0.0023, 0.0023])
    ops.mass(1954, *[0.0021, 0.0021, 0.0021])
    ops.mass(1955, *[0.0045, 0.0045, 0.0045])
    ops.mass(1956, *[0.0011, 0.0011, 0.0011])
    ops.mass(1957, *[0.0007, 0.0007, 0.0007])
    ops.mass(1958, *[0.002, 0.002, 0.002])
    ops.mass(1959, *[0.0012, 0.0012, 0.0012])
    ops.mass(1960, *[0.0014, 0.0014, 0.0014])
    ops.mass(1961, *[0.0021, 0.0021, 0.0021])
    ops.mass(1962, *[0.0013, 0.0013, 0.0013])
    ops.mass(1963, *[0.0015, 0.0015, 0.0015])
    ops.mass(1964, *[0.0019, 0.0019, 0.0019])
    ops.mass(1965, *[0.0003, 0.0003, 0.0003])
    ops.mass(1966, *[0.0009, 0.0009, 0.0009])
    ops.mass(1967, *[0.0007, 0.0007, 0.0007])
    ops.mass(1968, *[0.001, 0.001, 0.001])
    ops.mass(1969, *[0.0028, 0.0028, 0.0028])
    ops.mass(1970, *[0.0043, 0.0043, 0.0043])
    ops.mass(1971, *[0.0021, 0.0021, 0.0021])
    ops.mass(1972, *[0.0028, 0.0028, 0.0028])
    ops.mass(1973, *[0.0027, 0.0027, 0.0027])
    ops.mass(1974, *[0.0028, 0.0028, 0.0028])
    ops.mass(1975, *[0.0035, 0.0035, 0.0035])
    ops.mass(1976, *[0.0022, 0.0022, 0.0022])
    ops.mass(1977, *[0.0012, 0.0012, 0.0012])
    ops.mass(1978, *[0.0016, 0.0016, 0.0016])
    ops.mass(1979, *[0.0017, 0.0017, 0.0017])
    ops.mass(1980, *[0.0066, 0.0066, 0.0066])
    ops.mass(1981, *[0.0074, 0.0074, 0.0074])
    ops.mass(1982, *[0.0007, 0.0007, 0.0007])
    ops.mass(1983, *[0.0005, 0.0005, 0.0005])
    ops.mass(1984, *[0.0008, 0.0008, 0.0008])
    ops.mass(1985, *[0.0005, 0.0005, 0.0005])
    ops.mass(1986, *[0.0011, 0.0011, 0.0011])
    ops.mass(1987, *[0.0007, 0.0007, 0.0007])
    ops.mass(1988, *[0.0015, 0.0015, 0.0015])
    ops.mass(1989, *[0.001, 0.001, 0.001])
    ops.mass(1990, *[0.0009, 0.0009, 0.0009])
    ops.mass(1991, *[0.0006, 0.0006, 0.0006])
    ops.mass(1992, *[0.0019, 0.0019, 0.0019])
    ops.mass(1993, *[0.0024, 0.0024, 0.0024])
    ops.mass(1994, *[0.0017, 0.0017, 0.0017])
    ops.mass(1995, *[0.0014, 0.0014, 0.0014])
    ops.mass(1996, *[0.0018, 0.0018, 0.0018])
    ops.mass(1997, *[0.0006, 0.0006, 0.0006])
    ops.mass(1998, *[0.0013, 0.0013, 0.0013])
    ops.mass(1999, *[0.0019, 0.0019, 0.0019])
    ops.mass(2000, *[0.0026, 0.0026, 0.0026])
    ops.mass(2001, *[0.0025, 0.0025, 0.0025])
    ops.mass(2002, *[0.0028, 0.0028, 0.0028])
    ops.mass(2003, *[0.0016, 0.0016, 0.0016])
    ops.mass(2004, *[0.0024, 0.0024, 0.0024])
    ops.mass(2005, *[0.001, 0.001, 0.001])
    ops.mass(2006, *[0.0018, 0.0018, 0.0018])
    ops.mass(2007, *[0.0021, 0.0021, 0.0021])
    ops.mass(2008, *[0.0015, 0.0015, 0.0015])
    ops.mass(2009, *[0.002, 0.002, 0.002])
    ops.mass(2010, *[0.0015, 0.0015, 0.0015])
    ops.mass(2011, *[0.0009, 0.0009, 0.0009])
    ops.mass(2012, *[0.0012, 0.0012, 0.0012])
    ops.mass(2013, *[0.0022, 0.0022, 0.0022])
    ops.mass(2014, *[0.0016, 0.0016, 0.0016])
    ops.mass(2015, *[0.0012, 0.0012, 0.0012])
    ops.mass(2016, *[0.002, 0.002, 0.002])
    ops.mass(2017, *[0.0015, 0.0015, 0.0015])
    ops.mass(2018, *[0.0013, 0.0013, 0.0013])
    ops.mass(2019, *[0.0023, 0.0023, 0.0023])
    ops.mass(2020, *[0.0014, 0.0014, 0.0014])
    ops.mass(2021, *[0.0016, 0.0016, 0.0016])
    ops.mass(2022, *[0.0011, 0.0011, 0.0011])
    ops.mass(2023, *[0.0004, 0.0004, 0.0004])
    ops.mass(2024, *[0.0011, 0.0011, 0.0011])
    ops.mass(2025, *[0.0017, 0.0017, 0.0017])
    ops.mass(2026, *[0.0017, 0.0017, 0.0017])
    ops.mass(2027, *[0.0021, 0.0021, 0.0021])
    ops.mass(2028, *[0.0001, 0.0001, 0.0001])
    ops.mass(2029, *[0.0002, 0.0002, 0.0002])
    ops.mass(2030, *[0.001, 0.001, 0.001])
    ops.mass(2031, *[0.0042, 0.0042, 0.0042])
    ops.mass(2032, *[0.0011, 0.0011, 0.0011])
    ops.mass(2033, *[0.0019, 0.0019, 0.0019])
    ops.mass(2034, *[0.0005, 0.0005, 0.0005])
    ops.mass(2035, *[0.0019, 0.0019, 0.0019])
    ops.mass(2036, *[0.0008, 0.0008, 0.0008])
    ops.mass(2037, *[0.001, 0.001, 0.001])
    ops.mass(2038, *[0.0005, 0.0005, 0.0005])
    ops.mass(2039, *[0.0004, 0.0004, 0.0004])
    ops.mass(2040, *[0.0004, 0.0004, 0.0004])
    ops.mass(2041, *[0.0004, 0.0004, 0.0004])
    ops.mass(2042, *[0.0004, 0.0004, 0.0004])
    ops.mass(2043, *[0.0012, 0.0012, 0.0012])
    ops.mass(2044, *[0.0013, 0.0013, 0.0013])
    ops.mass(2045, *[0.0009, 0.0009, 0.0009])
    ops.mass(2046, *[0.0023, 0.0023, 0.0023])
    ops.mass(2047, *[0.0012, 0.0012, 0.0012])
    ops.mass(2048, *[0.003, 0.003, 0.003])
    ops.mass(2049, *[0.0014, 0.0014, 0.0014])
    ops.mass(2050, *[0.0011, 0.0011, 0.0011])
    ops.mass(2051, *[0.0026, 0.0026, 0.0026])
    ops.mass(2052, *[0.0014, 0.0014, 0.0014])
    ops.mass(2053, *[0.0012, 0.0012, 0.0012])
    ops.mass(2054, *[0.0015, 0.0015, 0.0015])
    ops.mass(2055, *[0.0077, 0.0077, 0.0077])
    ops.mass(2056, *[0.0009, 0.0009, 0.0009])
    ops.mass(2057, *[0.0012, 0.0012, 0.0012])
    ops.mass(2058, *[0.0019, 0.0019, 0.0019])
    ops.mass(2059, *[0.0016, 0.0016, 0.0016])
    ops.mass(2060, *[0.0011, 0.0011, 0.0011])
    ops.mass(2061, *[0.0016, 0.0016, 0.0016])
    ops.mass(2062, *[0.0016, 0.0016, 0.0016])
    ops.mass(2063, *[0.0016, 0.0016, 0.0016])
    ops.mass(2064, *[0.0014, 0.0014, 0.0014])
    ops.mass(2065, *[0.0019, 0.0019, 0.0019])
    ops.mass(2066, *[0.0008, 0.0008, 0.0008])
    ops.mass(2067, *[0.0008, 0.0008, 0.0008])
    ops.mass(2068, *[0.0027, 0.0027, 0.0027])
    ops.mass(2069, *[0.001, 0.001, 0.001])
    ops.mass(2070, *[0.0022, 0.0022, 0.0022])
    ops.mass(2071, *[0.002, 0.002, 0.002])
    ops.mass(2072, *[0.0023, 0.0023, 0.0023])
    ops.mass(2073, *[0.0032, 0.0032, 0.0032])
    ops.mass(2074, *[0.0022, 0.0022, 0.0022])
    ops.mass(2075, *[0.0013, 0.0013, 0.0013])
    ops.mass(2076, *[0.0019, 0.0019, 0.0019])
    ops.mass(2077, *[0.0013, 0.0013, 0.0013])
    ops.mass(2078, *[0.002, 0.002, 0.002])
    ops.mass(2079, *[0.0021, 0.0021, 0.0021])
    ops.mass(2080, *[0.0022, 0.0022, 0.0022])
    ops.mass(2081, *[0.0015, 0.0015, 0.0015])
    ops.mass(2082, *[0.001, 0.001, 0.001])
    ops.mass(2083, *[0.0018, 0.0018, 0.0018])
    ops.mass(2084, *[0.0012, 0.0012, 0.0012])
    ops.mass(2085, *[0.0012, 0.0012, 0.0012])
    ops.mass(2086, *[0.0021, 0.0021, 0.0021])
    ops.mass(2087, *[0.0007, 0.0007, 0.0007])
    ops.mass(2088, *[0.0021, 0.0021, 0.0021])
    ops.mass(2089, *[0.001, 0.001, 0.001])
    ops.mass(2090, *[0.0008, 0.0008, 0.0008])
    ops.mass(2091, *[0.001, 0.001, 0.001])
    ops.mass(2092, *[0.0008, 0.0008, 0.0008])
    ops.mass(2093, *[0.0015, 0.0015, 0.0015])
    ops.mass(2094, *[0.0012, 0.0012, 0.0012])
    ops.mass(2095, *[0.0018, 0.0018, 0.0018])
    ops.mass(2096, *[0.0011, 0.0011, 0.0011])
    ops.mass(2097, *[0.0017, 0.0017, 0.0017])
    ops.mass(2098, *[0.0034, 0.0034, 0.0034])
    ops.mass(2099, *[0.002, 0.002, 0.002])
    ops.mass(2100, *[0.0013, 0.0013, 0.0013])
    ops.mass(2101, *[0.001, 0.001, 0.001])
    ops.mass(2102, *[0.0014, 0.0014, 0.0014])
    ops.mass(2103, *[0.0017, 0.0017, 0.0017])
    ops.mass(2104, *[0.0012, 0.0012, 0.0012])
    ops.mass(2105, *[0.0005, 0.0005, 0.0005])
    ops.mass(2106, *[0.001, 0.001, 0.001])
    ops.mass(2107, *[0.002, 0.002, 0.002])
    ops.mass(2108, *[0.0012, 0.0012, 0.0012])
    ops.mass(2109, *[0.0011, 0.0011, 0.0011])
    ops.mass(2110, *[0.0015, 0.0015, 0.0015])
    ops.mass(2111, *[0.0022, 0.0022, 0.0022])
    ops.mass(2112, *[0.001, 0.001, 0.001])
    ops.mass(2113, *[0.0019, 0.0019, 0.0019])
    ops.mass(2114, *[0.0044, 0.0044, 0.0044])
    ops.mass(2115, *[0.0007, 0.0007, 0.0007])
    ops.mass(2116, *[0.0013, 0.0013, 0.0013])
    ops.mass(2117, *[0.0008, 0.0008, 0.0008])
    ops.mass(2118, *[0.0006, 0.0006, 0.0006])
    ops.mass(2119, *[0.0005, 0.0005, 0.0005])
    ops.mass(2120, *[0.0031, 0.0031, 0.0031])
    ops.mass(2121, *[0.0013, 0.0013, 0.0013])
    ops.mass(2122, *[0.0007, 0.0007, 0.0007])
    ops.mass(2123, *[0.0017, 0.0017, 0.0017])
    ops.mass(2124, *[0.0007, 0.0007, 0.0007])
    ops.mass(2125, *[0.0016, 0.0016, 0.0016])
    ops.mass(2126, *[0.0005, 0.0005, 0.0005])
    ops.mass(2127, *[0.0005, 0.0005, 0.0005])
    ops.mass(2128, *[0.0005, 0.0005, 0.0005])
    ops.mass(2129, *[0.0021, 0.0021, 0.0021])
    ops.mass(2130, *[0.0013, 0.0013, 0.0013])
    ops.mass(2131, *[0.0009, 0.0009, 0.0009])
    ops.mass(2132, *[0.0013, 0.0013, 0.0013])
    ops.mass(2133, *[0.0031, 0.0031, 0.0031])
    ops.mass(2134, *[0.003, 0.003, 0.003])
    ops.mass(2135, *[0.0034, 0.0034, 0.0034])
    ops.mass(2136, *[0.0004, 0.0004, 0.0004])
    ops.mass(2137, *[0.0007, 0.0007, 0.0007])
    ops.mass(2138, *[0.0016, 0.0016, 0.0016])
    ops.mass(2139, *[0.0023, 0.0023, 0.0023])
    ops.mass(2140, *[0.0029, 0.0029, 0.0029])
    ops.mass(2141, *[0.0001, 0.0001, 0.0001])
    ops.mass(2142, *[0.0023, 0.0023, 0.0023])
    ops.mass(2143, *[0.0025, 0.0025, 0.0025])
    ops.mass(2144, *[0.0026, 0.0026, 0.0026])
    ops.mass(2145, *[0.0033, 0.0033, 0.0033])
    ops.mass(2146, *[0.0012, 0.0012, 0.0012])
    ops.mass(2147, *[0.0034, 0.0034, 0.0034])
    ops.mass(2148, *[0.001, 0.001, 0.001])
    ops.mass(2149, *[0.0008, 0.0008, 0.0008])
    ops.mass(2150, *[0.0017, 0.0017, 0.0017])
    ops.mass(2151, *[0.0022, 0.0022, 0.0022])
    ops.mass(2152, *[0.0015, 0.0015, 0.0015])
    ops.mass(2153, *[0.0023, 0.0023, 0.0023])
    ops.mass(2154, *[0.0019, 0.0019, 0.0019])
    ops.mass(2155, *[0.002, 0.002, 0.002])
    ops.mass(2156, *[0.0006, 0.0006, 0.0006])
    ops.mass(2157, *[0.0025, 0.0025, 0.0025])
    ops.mass(2158, *[0.0012, 0.0012, 0.0012])
    ops.mass(2159, *[0.0007, 0.0007, 0.0007])
    ops.mass(2160, *[0.0021, 0.0021, 0.0021])
    ops.mass(2161, *[0.0005, 0.0005, 0.0005])
    ops.mass(2162, *[0.0007, 0.0007, 0.0007])
    ops.mass(2163, *[0.0005, 0.0005, 0.0005])
    ops.mass(2164, *[0.0007, 0.0007, 0.0007])
    ops.mass(2165, *[0.0008, 0.0008, 0.0008])
    ops.mass(2166, *[0.001, 0.001, 0.001])
    ops.mass(2167, *[0.0028, 0.0028, 0.0028])
    ops.mass(2168, *[0.0024, 0.0024, 0.0024])
    ops.mass(2169, *[0.0026, 0.0026, 0.0026])
    ops.mass(2170, *[0.0039, 0.0039, 0.0039])
    ops.mass(2171, *[0.0061, 0.0061, 0.0061])
    ops.mass(2172, *[0.0027, 0.0027, 0.0027])
    ops.mass(2173, *[0.002, 0.002, 0.002])
    ops.mass(2174, *[0.0016, 0.0016, 0.0016])
    ops.mass(2175, *[0.0015, 0.0015, 0.0015])
    ops.mass(2176, *[0.0013, 0.0013, 0.0013])
    ops.mass(2177, *[0.0004, 0.0004, 0.0004])
    ops.mass(2178, *[0.001, 0.001, 0.001])
    ops.mass(2179, *[0.0008, 0.0008, 0.0008])
    ops.mass(2180, *[0.0003, 0.0003, 0.0003])
    ops.mass(2181, *[0.0006, 0.0006, 0.0006])
    ops.mass(2182, *[0.0006, 0.0006, 0.0006])
    ops.mass(2183, *[0.0004, 0.0004, 0.0004])
    ops.mass(2184, *[0.0022, 0.0022, 0.0022])
    ops.mass(2185, *[0.0021, 0.0021, 0.0021])
    ops.mass(2186, *[0.002, 0.002, 0.002])
    ops.mass(2187, *[0.0012, 0.0012, 0.0012])
    ops.mass(2188, *[0.001, 0.001, 0.001])
    ops.mass(2189, *[0.0009, 0.0009, 0.0009])
    ops.mass(2190, *[0.0008, 0.0008, 0.0008])
    ops.mass(2191, *[0.0008, 0.0008, 0.0008])
    ops.mass(2192, *[0.0016, 0.0016, 0.0016])
    ops.mass(2193, *[0.0017, 0.0017, 0.0017])
    ops.mass(2194, *[0.0021, 0.0021, 0.0021])
    ops.mass(2195, *[0.0013, 0.0013, 0.0013])
    ops.mass(2196, *[0.0006, 0.0006, 0.0006])
    ops.mass(2197, *[0.0004, 0.0004, 0.0004])
    ops.mass(2198, *[0.0003, 0.0003, 0.0003])
    ops.mass(2199, *[0.0019, 0.0019, 0.0019])
    ops.mass(2200, *[0.0019, 0.0019, 0.0019])
    ops.mass(2201, *[0.0004, 0.0004, 0.0004])
    ops.mass(2202, *[0.0022, 0.0022, 0.0022])
    ops.mass(2203, *[0.002, 0.002, 0.002])
    ops.mass(2204, *[0.0032, 0.0032, 0.0032])
    ops.mass(2205, *[0.0005, 0.0005, 0.0005])
    ops.mass(2206, *[0.0004, 0.0004, 0.0004])
    ops.mass(2207, *[0.0005, 0.0005, 0.0005])
    ops.mass(2208, *[0.0007, 0.0007, 0.0007])
    ops.mass(2209, *[0.0014, 0.0014, 0.0014])
    ops.mass(2210, *[0.0015, 0.0015, 0.0015])
    ops.mass(2211, *[0.002, 0.002, 0.002])
    ops.mass(2212, *[0.0003, 0.0003, 0.0003])
    ops.mass(2213, *[0.0018, 0.0018, 0.0018])
    ops.mass(2214, *[0.0047, 0.0047, 0.0047])
    ops.mass(2215, *[0.0016, 0.0016, 0.0016])
    ops.mass(2216, *[0.0006, 0.0006, 0.0006])
    ops.mass(2217, *[0.0002, 0.0002, 0.0002])
    ops.mass(2218, *[0.0018, 0.0018, 0.0018])
    ops.mass(2219, *[0.0021, 0.0021, 0.0021])
    ops.mass(2220, *[0.0016, 0.0016, 0.0016])
    ops.mass(2221, *[0.0023, 0.0023, 0.0023])
    ops.mass(2222, *[0.0022, 0.0022, 0.0022])
    ops.mass(2223, *[0.0013, 0.0013, 0.0013])
    ops.mass(2224, *[0.0016, 0.0016, 0.0016])
    ops.mass(2225, *[0.0028, 0.0028, 0.0028])
    ops.mass(2226, *[0.0019, 0.0019, 0.0019])
    ops.mass(2227, *[0.0016, 0.0016, 0.0016])
    ops.mass(2228, *[0.0013, 0.0013, 0.0013])
    ops.mass(2229, *[0.0027, 0.0027, 0.0027])
    ops.mass(2230, *[0.0017, 0.0017, 0.0017])
    ops.mass(2231, *[0.0016, 0.0016, 0.0016])
    ops.mass(2232, *[0.0013, 0.0013, 0.0013])
    ops.mass(2233, *[0.0019, 0.0019, 0.0019])
    ops.mass(2234, *[0.0018, 0.0018, 0.0018])
    ops.mass(2235, *[0.0003, 0.0003, 0.0003])
    ops.mass(2236, *[0.005, 0.005, 0.005])
    ops.mass(2237, *[0.0066, 0.0066, 0.0066])
    ops.mass(2238, *[0.0077, 0.0077, 0.0077])
    ops.mass(2239, *[0.0012, 0.0012, 0.0012])
    ops.mass(2240, *[0.0022, 0.0022, 0.0022])
    ops.mass(2241, *[0.0023, 0.0023, 0.0023])
    ops.mass(2242, *[0.0023, 0.0023, 0.0023])
    ops.mass(2243, *[0.0022, 0.0022, 0.0022])
    ops.mass(2244, *[0.0015, 0.0015, 0.0015])
    ops.mass(2245, *[0.0021, 0.0021, 0.0021])
    ops.mass(2246, *[0.0006, 0.0006, 0.0006])
    ops.mass(2247, *[0.0007, 0.0007, 0.0007])
    ops.mass(2248, *[0.0024, 0.0024, 0.0024])
    ops.mass(2249, *[0.0007, 0.0007, 0.0007])
    ops.mass(2250, *[0.0019, 0.0019, 0.0019])
    ops.mass(2251, *[0.0007, 0.0007, 0.0007])
    ops.mass(2252, *[0.0008, 0.0008, 0.0008])
    ops.mass(2253, *[0.0007, 0.0007, 0.0007])
    ops.mass(2254, *[0.0003, 0.0003, 0.0003])
    ops.mass(2255, *[0.0026, 0.0026, 0.0026])
    ops.mass(2256, *[0.0017, 0.0017, 0.0017])
    ops.mass(2257, *[0.0007, 0.0007, 0.0007])
    ops.mass(2258, *[0.0016, 0.0016, 0.0016])
    ops.mass(2259, *[0.0001, 0.0001, 0.0001])
    ops.mass(2260, *[0.0015, 0.0015, 0.0015])
    ops.mass(2261, *[0.0015, 0.0015, 0.0015])
    ops.mass(2262, *[0.0005, 0.0005, 0.0005])
    ops.mass(2263, *[0.0012, 0.0012, 0.0012])
    ops.mass(2264, *[0.0017, 0.0017, 0.0017])
    ops.mass(2265, *[0.0005, 0.0005, 0.0005])
    ops.mass(2266, *[0.0017, 0.0017, 0.0017])
    ops.mass(2267, *[0.0007, 0.0007, 0.0007])
    ops.mass(2268, *[0.0006, 0.0006, 0.0006])
    ops.mass(2269, *[0.001, 0.001, 0.001])
    ops.mass(2270, *[0.0007, 0.0007, 0.0007])
    ops.mass(2271, *[0.0009, 0.0009, 0.0009])
    ops.mass(2272, *[0.0016, 0.0016, 0.0016])
    ops.mass(2273, *[0.002, 0.002, 0.002])
    ops.mass(2274, *[0.0016, 0.0016, 0.0016])
    ops.mass(2275, *[0.0018, 0.0018, 0.0018])
    ops.mass(2276, *[0.0005, 0.0005, 0.0005])
    ops.mass(2277, *[0.0012, 0.0012, 0.0012])
    ops.mass(2278, *[0.0008, 0.0008, 0.0008])
    ops.mass(2279, *[0.0008, 0.0008, 0.0008])
    ops.mass(2280, *[0.0013, 0.0013, 0.0013])
    ops.mass(2281, *[0.0006, 0.0006, 0.0006])
    ops.mass(2282, *[0.0007, 0.0007, 0.0007])
    ops.mass(2283, *[0.002, 0.002, 0.002])
    ops.mass(2284, *[0.0022, 0.0022, 0.0022])
    ops.mass(2285, *[0.0004, 0.0004, 0.0004])
    ops.mass(2286, *[0.0002, 0.0002, 0.0002])
    ops.mass(2287, *[0.0006, 0.0006, 0.0006])
    ops.mass(2288, *[0.0004, 0.0004, 0.0004])
    ops.mass(2289, *[0.0005, 0.0005, 0.0005])
    ops.mass(2290, *[0.0017, 0.0017, 0.0017])
    ops.mass(2291, *[0.0006, 0.0006, 0.0006])
    ops.mass(2292, *[0.0018, 0.0018, 0.0018])
    ops.mass(2293, *[0.0005, 0.0005, 0.0005])
    ops.mass(2294, *[0.0017, 0.0017, 0.0017])
    ops.mass(2295, *[0.0009, 0.0009, 0.0009])
    ops.mass(2296, *[0.0008, 0.0008, 0.0008])
    ops.mass(2297, *[0.0012, 0.0012, 0.0012])
    ops.mass(2298, *[0.0021, 0.0021, 0.0021])
    ops.mass(2299, *[0.0008, 0.0008, 0.0008])
    ops.mass(2300, *[0.0013, 0.0013, 0.0013])
    ops.mass(2301, *[0.002, 0.002, 0.002])
    ops.mass(2302, *[0.0013, 0.0013, 0.0013])
    ops.mass(2303, *[0.0016, 0.0016, 0.0016])
    ops.mass(2304, *[0.0011, 0.0011, 0.0011])
    ops.mass(2305, *[0.0008, 0.0008, 0.0008])
    ops.mass(2306, *[0.0019, 0.0019, 0.0019])
    ops.mass(2307, *[0.002, 0.002, 0.002])
    ops.mass(2308, *[0.003, 0.003, 0.003])
    ops.mass(2309, *[0.0021, 0.0021, 0.0021])
    ops.mass(2310, *[0.0019, 0.0019, 0.0019])
    ops.mass(2311, *[0.0019, 0.0019, 0.0019])
    ops.mass(2312, *[0.0013, 0.0013, 0.0013])
    ops.mass(2313, *[0.0006, 0.0006, 0.0006])
    ops.mass(2314, *[0.0003, 0.0003, 0.0003])
    ops.mass(2315, *[0.002, 0.002, 0.002])
    ops.mass(2316, *[0.0019, 0.0019, 0.0019])
    ops.mass(2317, *[0.0007, 0.0007, 0.0007])
    ops.mass(2318, *[0.0008, 0.0008, 0.0008])
    ops.mass(2319, *[0.0014, 0.0014, 0.0014])
    ops.mass(2320, *[0.0007, 0.0007, 0.0007])
    ops.mass(2321, *[0.0026, 0.0026, 0.0026])
    ops.mass(2322, *[0.001, 0.001, 0.001])
    ops.mass(2323, *[0.0006, 0.0006, 0.0006])
    ops.mass(2324, *[0.0008, 0.0008, 0.0008])
    ops.mass(2325, *[0.0024, 0.0024, 0.0024])
    ops.mass(2326, *[0.0003, 0.0003, 0.0003])
    ops.mass(2327, *[0.0004, 0.0004, 0.0004])
    ops.mass(2328, *[0.0007, 0.0007, 0.0007])
    ops.mass(2329, *[0.001, 0.001, 0.001])
    ops.mass(2330, *[0.0012, 0.0012, 0.0012])
    ops.mass(2331, *[0.0016, 0.0016, 0.0016])
    ops.mass(2332, *[0.0027, 0.0027, 0.0027])
    ops.mass(2333, *[0.0021, 0.0021, 0.0021])
    ops.mass(2334, *[0.0019, 0.0019, 0.0019])
    ops.mass(2335, *[0.0015, 0.0015, 0.0015])
    ops.mass(2336, *[0.0014, 0.0014, 0.0014])
    ops.mass(2337, *[0.0015, 0.0015, 0.0015])
    ops.mass(2338, *[0.0063, 0.0063, 0.0063])
    ops.mass(2339, *[0.0013, 0.0013, 0.0013])
    ops.mass(2340, *[0.0044, 0.0044, 0.0044])
    ops.mass(2341, *[0.0007, 0.0007, 0.0007])
    ops.mass(2342, *[0.001, 0.001, 0.001])
    ops.mass(2343, *[0.0011, 0.0011, 0.0011])
    ops.mass(2344, *[0.0009, 0.0009, 0.0009])
    ops.mass(2345, *[0.0012, 0.0012, 0.0012])
    ops.mass(2346, *[0.0008, 0.0008, 0.0008])
    ops.mass(2347, *[0.0015, 0.0015, 0.0015])
    ops.mass(2348, *[0.002, 0.002, 0.002])
    ops.mass(2349, *[0.003, 0.003, 0.003])
    ops.mass(2350, *[0.0023, 0.0023, 0.0023])
    ops.mass(2351, *[0.0025, 0.0025, 0.0025])
    ops.mass(2352, *[0.0012, 0.0012, 0.0012])
    ops.mass(2353, *[0.0005, 0.0005, 0.0005])
    ops.mass(2354, *[0.0008, 0.0008, 0.0008])
    ops.mass(2355, *[0.0007, 0.0007, 0.0007])
    ops.mass(2356, *[0.0019, 0.0019, 0.0019])
    ops.mass(2357, *[0.0124, 0.0124, 0.0124])
    ops.mass(2358, *[0.0102, 0.0102, 0.0102])
    ops.mass(2359, *[0.0021, 0.0021, 0.0021])
    ops.mass(2360, *[0.0007, 0.0007, 0.0007])
    ops.mass(2361, *[0.0013, 0.0013, 0.0013])
    ops.mass(2362, *[0.0016, 0.0016, 0.0016])
    ops.mass(2363, *[0.0006, 0.0006, 0.0006])
    ops.mass(2364, *[0.0016, 0.0016, 0.0016])
    ops.mass(2365, *[0.0008, 0.0008, 0.0008])
    ops.mass(2366, *[0.0009, 0.0009, 0.0009])
    ops.mass(2367, *[0.0002, 0.0002, 0.0002])
    ops.mass(2368, *[0.0019, 0.0019, 0.0019])
    ops.mass(2369, *[0.0014, 0.0014, 0.0014])
    ops.mass(2370, *[0.0006, 0.0006, 0.0006])
    ops.mass(2371, *[0.0005, 0.0005, 0.0005])
    ops.mass(2372, *[0.0005, 0.0005, 0.0005])
    ops.mass(2373, *[0.0019, 0.0019, 0.0019])
    ops.mass(2374, *[0.0004, 0.0004, 0.0004])
    ops.mass(2375, *[0.0016, 0.0016, 0.0016])
    ops.mass(2376, *[0.0014, 0.0014, 0.0014])
    ops.mass(2377, *[0.0011, 0.0011, 0.0011])
    ops.mass(2378, *[0.0005, 0.0005, 0.0005])
    ops.mass(2379, *[0.0011, 0.0011, 0.0011])
    ops.mass(2380, *[0.0015, 0.0015, 0.0015])
    ops.mass(2381, *[0.0051, 0.0051, 0.0051])
    ops.mass(2382, *[0.0019, 0.0019, 0.0019])
    ops.mass(2383, *[0.0026, 0.0026, 0.0026])
    ops.mass(2384, *[0.0014, 0.0014, 0.0014])
    ops.mass(2385, *[0.0014, 0.0014, 0.0014])
    ops.mass(2386, *[0.0017, 0.0017, 0.0017])
    ops.mass(2387, *[0.0016, 0.0016, 0.0016])
    ops.mass(2388, *[0.0016, 0.0016, 0.0016])
    ops.mass(2389, *[0.0015, 0.0015, 0.0015])
    ops.mass(2390, *[0.0018, 0.0018, 0.0018])
    ops.mass(2391, *[0.0019, 0.0019, 0.0019])
    ops.mass(2392, *[0.0012, 0.0012, 0.0012])
    ops.mass(2393, *[0.0006, 0.0006, 0.0006])
    ops.mass(2394, *[0.0008, 0.0008, 0.0008])
    ops.mass(2395, *[0.0024, 0.0024, 0.0024])
    ops.mass(2396, *[0.0046, 0.0046, 0.0046])
    ops.mass(2397, *[0.0017, 0.0017, 0.0017])
    ops.mass(2398, *[0.0016, 0.0016, 0.0016])
    ops.mass(2399, *[0.0012, 0.0012, 0.0012])
    ops.mass(2400, *[0.0011, 0.0011, 0.0011])
    ops.mass(2401, *[0.0008, 0.0008, 0.0008])
    ops.mass(2402, *[0.004, 0.004, 0.004])
    ops.mass(2403, *[0.0007, 0.0007, 0.0007])
    ops.mass(2404, *[0.0024, 0.0024, 0.0024])
    ops.mass(2405, *[0.0033, 0.0033, 0.0033])
    ops.mass(2406, *[0.0027, 0.0027, 0.0027])
    ops.mass(2407, *[0.0016, 0.0016, 0.0016])
    ops.mass(2408, *[0.0033, 0.0033, 0.0033])
    ops.mass(2409, *[0.0026, 0.0026, 0.0026])
    ops.mass(2410, *[0.0018, 0.0018, 0.0018])
    ops.mass(2411, *[0.0024, 0.0024, 0.0024])
    ops.mass(2412, *[0.0024, 0.0024, 0.0024])
    ops.mass(2413, *[0.0024, 0.0024, 0.0024])
    ops.mass(2414, *[0.0009, 0.0009, 0.0009])
    ops.mass(2415, *[0.0006, 0.0006, 0.0006])
    ops.mass(2416, *[0.0024, 0.0024, 0.0024])
    ops.mass(2417, *[0.0004, 0.0004, 0.0004])
    ops.mass(2418, *[0.0006, 0.0006, 0.0006])
    ops.mass(2419, *[0.0011, 0.0011, 0.0011])
    ops.mass(2420, *[0.002, 0.002, 0.002])
    ops.mass(2421, *[0.0009, 0.0009, 0.0009])
    ops.mass(2422, *[0.0025, 0.0025, 0.0025])
    ops.mass(2423, *[0.0001, 0.0001, 0.0001])
    ops.mass(2424, *[0.0011, 0.0011, 0.0011])
    ops.mass(2425, *[0.0008, 0.0008, 0.0008])
    ops.mass(2426, *[0.0004, 0.0004, 0.0004])
    ops.mass(2427, *[0.0005, 0.0005, 0.0005])
    ops.mass(2428, *[0.0011, 0.0011, 0.0011])
    ops.mass(2429, *[0.0015, 0.0015, 0.0015])
    ops.mass(2430, *[0.0016, 0.0016, 0.0016])
    ops.mass(2431, *[0.0005, 0.0005, 0.0005])
    ops.mass(2432, *[0.0009, 0.0009, 0.0009])
    ops.mass(2433, *[0.0009, 0.0009, 0.0009])
    ops.mass(2434, *[0.0008, 0.0008, 0.0008])
    ops.mass(2435, *[0.0022, 0.0022, 0.0022])
    ops.mass(2436, *[0.0016, 0.0016, 0.0016])
    ops.mass(2437, *[0.0005, 0.0005, 0.0005])
    ops.mass(2438, *[0.007, 0.007, 0.007])
    ops.mass(2439, *[0.0007, 0.0007, 0.0007])
    ops.mass(2440, *[0.0016, 0.0016, 0.0016])
    ops.mass(2441, *[0.0006, 0.0006, 0.0006])
    ops.mass(2442, *[0.0013, 0.0013, 0.0013])
    ops.mass(2443, *[0.002, 0.002, 0.002])
    ops.mass(2444, *[0.0007, 0.0007, 0.0007])
    ops.mass(2445, *[0.0005, 0.0005, 0.0005])
    ops.mass(2446, *[0.0005, 0.0005, 0.0005])
    ops.mass(2447, *[0.0006, 0.0006, 0.0006])
    ops.mass(2448, *[0.0003, 0.0003, 0.0003])
    ops.mass(2449, *[0.0009, 0.0009, 0.0009])
    ops.mass(2450, *[0.0013, 0.0013, 0.0013])
    ops.mass(2451, *[0.0013, 0.0013, 0.0013])
    ops.mass(2452, *[0.0012, 0.0012, 0.0012])
    ops.mass(2453, *[0.0014, 0.0014, 0.0014])
    ops.mass(2454, *[0.0006, 0.0006, 0.0006])
    ops.mass(2455, *[0.0006, 0.0006, 0.0006])
    ops.mass(2456, *[0.0006, 0.0006, 0.0006])
    ops.mass(2457, *[0.0015, 0.0015, 0.0015])
    ops.mass(2458, *[0.0005, 0.0005, 0.0005])
    ops.mass(2459, *[0.0007, 0.0007, 0.0007])
    ops.mass(2460, *[0.0007, 0.0007, 0.0007])
    ops.mass(2461, *[0.0007, 0.0007, 0.0007])
    ops.mass(2462, *[0.0027, 0.0027, 0.0027])
    ops.mass(2463, *[0.0009, 0.0009, 0.0009])
    ops.mass(2464, *[0.0018, 0.0018, 0.0018])
    ops.mass(2465, *[0.002, 0.002, 0.002])
    ops.mass(2466, *[0.0011, 0.0011, 0.0011])
    ops.mass(2467, *[0.002, 0.002, 0.002])
    ops.mass(2468, *[0.0013, 0.0013, 0.0013])
    ops.mass(2469, *[0.0003, 0.0003, 0.0003])
    ops.mass(2470, *[0.0006, 0.0006, 0.0006])
    ops.mass(2471, *[0.0006, 0.0006, 0.0006])
    ops.mass(2472, *[0.0009, 0.0009, 0.0009])
    ops.mass(2473, *[0.0005, 0.0005, 0.0005])
    ops.mass(2474, *[0.0008, 0.0008, 0.0008])
    ops.mass(2475, *[0.0018, 0.0018, 0.0018])
    ops.mass(2476, *[0.0006, 0.0006, 0.0006])
    ops.mass(2477, *[0.0003, 0.0003, 0.0003])
    ops.mass(2478, *[0.0027, 0.0027, 0.0027])
    ops.mass(2479, *[0.0079, 0.0079, 0.0079])
    ops.mass(2480, *[0.0005, 0.0005, 0.0005])
    ops.mass(2481, *[0.0003, 0.0003, 0.0003])
    ops.mass(2482, *[0.002, 0.002, 0.002])
    ops.mass(2483, *[0.0018, 0.0018, 0.0018])
    ops.mass(2484, *[0.0017, 0.0017, 0.0017])
    ops.mass(2485, *[0.0017, 0.0017, 0.0017])
    ops.mass(2486, *[0.0018, 0.0018, 0.0018])
    ops.mass(2487, *[0.0007, 0.0007, 0.0007])
    ops.mass(2488, *[0.0016, 0.0016, 0.0016])
    ops.mass(2489, *[0.0012, 0.0012, 0.0012])
    ops.mass(2490, *[0.0009, 0.0009, 0.0009])
    ops.mass(2491, *[0.0017, 0.0017, 0.0017])
    ops.mass(2492, *[0.0022, 0.0022, 0.0022])
    ops.mass(2493, *[0.0011, 0.0011, 0.0011])
    ops.mass(2494, *[0.002, 0.002, 0.002])
    ops.mass(2495, *[0.0014, 0.0014, 0.0014])
    ops.mass(2496, *[0.0018, 0.0018, 0.0018])
    ops.mass(2497, *[0.002, 0.002, 0.002])
    ops.mass(2498, *[0.0012, 0.0012, 0.0012])
    ops.mass(2499, *[0.0005, 0.0005, 0.0005])
    ops.mass(2500, *[0.0004, 0.0004, 0.0004])
    ops.mass(2501, *[0.0012, 0.0012, 0.0012])
    ops.mass(2502, *[0.0022, 0.0022, 0.0022])
    ops.mass(2503, *[0.0008, 0.0008, 0.0008])
    ops.mass(2504, *[0.0006, 0.0006, 0.0006])
    ops.mass(2505, *[0.0012, 0.0012, 0.0012])
    ops.mass(2506, *[0.0013, 0.0013, 0.0013])
    ops.mass(2507, *[0.0007, 0.0007, 0.0007])
    ops.mass(2508, *[0.0014, 0.0014, 0.0014])
    ops.mass(2509, *[0.0011, 0.0011, 0.0011])
    ops.mass(2510, *[0.0011, 0.0011, 0.0011])
    ops.mass(2511, *[0.0008, 0.0008, 0.0008])
    ops.mass(2512, *[0.0013, 0.0013, 0.0013])
    ops.mass(2513, *[0.0014, 0.0014, 0.0014])
    ops.mass(2514, *[0.0014, 0.0014, 0.0014])
    ops.mass(2515, *[0.0004, 0.0004, 0.0004])
    ops.mass(2516, *[0.0017, 0.0017, 0.0017])
    ops.mass(2517, *[0.0006, 0.0006, 0.0006])
    ops.mass(2518, *[0.0007, 0.0007, 0.0007])
    ops.mass(2519, *[0.0016, 0.0016, 0.0016])
    ops.mass(2520, *[0.0013, 0.0013, 0.0013])
    ops.mass(2521, *[0.0009, 0.0009, 0.0009])
    ops.mass(2522, *[0.0013, 0.0013, 0.0013])
    ops.mass(2523, *[0.0008, 0.0008, 0.0008])
    ops.mass(2524, *[0.0014, 0.0014, 0.0014])
    ops.mass(2525, *[0.0016, 0.0016, 0.0016])
    ops.mass(2526, *[0.002, 0.002, 0.002])
    ops.mass(2527, *[0.001, 0.001, 0.001])
    ops.mass(2528, *[0.0017, 0.0017, 0.0017])
    ops.mass(2529, *[0.0021, 0.0021, 0.0021])
    ops.mass(2530, *[0.0012, 0.0012, 0.0012])
    ops.mass(2531, *[0.0027, 0.0027, 0.0027])
    ops.mass(2532, *[0.0025, 0.0025, 0.0025])
    ops.mass(2533, *[0.0021, 0.0021, 0.0021])
    ops.mass(2534, *[0.0004, 0.0004, 0.0004])
    ops.mass(2535, *[0.0003, 0.0003, 0.0003])
    ops.mass(2536, *[0.0012, 0.0012, 0.0012])
    ops.mass(2537, *[0.0025, 0.0025, 0.0025])
    ops.mass(2538, *[0.0022, 0.0022, 0.0022])
    ops.mass(2539, *[0.0023, 0.0023, 0.0023])
    ops.mass(2540, *[0.0011, 0.0011, 0.0011])
    ops.mass(2541, *[0.0011, 0.0011, 0.0011])
    ops.mass(2542, *[0.0031, 0.0031, 0.0031])
    ops.mass(2543, *[0.0011, 0.0011, 0.0011])
    ops.mass(2544, *[0.0026, 0.0026, 0.0026])
    ops.mass(2545, *[0.0005, 0.0005, 0.0005])
    ops.mass(2546, *[0.0014, 0.0014, 0.0014])
    ops.mass(2547, *[0.0002, 0.0002, 0.0002])
    ops.mass(2548, *[0.0003, 0.0003, 0.0003])
    ops.mass(2549, *[0.0006, 0.0006, 0.0006])
    ops.mass(2550, *[0.0004, 0.0004, 0.0004])
    ops.mass(2551, *[0.0007, 0.0007, 0.0007])
    ops.mass(2552, *[0.0003, 0.0003, 0.0003])
    ops.mass(2553, *[0.001, 0.001, 0.001])
    ops.mass(2554, *[0.0013, 0.0013, 0.0013])
    ops.mass(2555, *[0.0014, 0.0014, 0.0014])
    ops.mass(2556, *[0.0022, 0.0022, 0.0022])
    ops.mass(2557, *[0.0008, 0.0008, 0.0008])
    ops.mass(2558, *[0.0011, 0.0011, 0.0011])
    ops.mass(2559, *[0.0018, 0.0018, 0.0018])
    ops.mass(2560, *[0.0057, 0.0057, 0.0057])
    ops.mass(2561, *[0.0008, 0.0008, 0.0008])
    ops.mass(2562, *[0.0025, 0.0025, 0.0025])
    ops.mass(2563, *[0.0021, 0.0021, 0.0021])
    ops.mass(2564, *[0.0008, 0.0008, 0.0008])
    ops.mass(2565, *[0.0012, 0.0012, 0.0012])
    ops.mass(2566, *[0.0007, 0.0007, 0.0007])
    ops.mass(2567, *[0.0003, 0.0003, 0.0003])
    ops.mass(2568, *[0.0015, 0.0015, 0.0015])
    ops.mass(2569, *[0.0004, 0.0004, 0.0004])
    ops.mass(2570, *[0.0043, 0.0043, 0.0043])
    ops.mass(2571, *[0.0008, 0.0008, 0.0008])
    ops.mass(2572, *[0.0006, 0.0006, 0.0006])
    ops.mass(2573, *[0.001, 0.001, 0.001])
    ops.mass(2574, *[0.0006, 0.0006, 0.0006])
    ops.mass(2575, *[0.0004, 0.0004, 0.0004])
    ops.mass(2576, *[0.0003, 0.0003, 0.0003])
    ops.mass(2577, *[0.0018, 0.0018, 0.0018])
    ops.mass(2578, *[0.0036, 0.0036, 0.0036])
    ops.mass(2579, *[0.0012, 0.0012, 0.0012])
    ops.mass(2580, *[0.0005, 0.0005, 0.0005])
    ops.mass(2581, *[0.0005, 0.0005, 0.0005])
    ops.mass(2582, *[0.0007, 0.0007, 0.0007])
    ops.mass(2583, *[0.0007, 0.0007, 0.0007])
    ops.mass(2584, *[0.0007, 0.0007, 0.0007])
    ops.mass(2585, *[0.0012, 0.0012, 0.0012])
    ops.mass(2586, *[0.0004, 0.0004, 0.0004])
    ops.mass(2587, *[0.0004, 0.0004, 0.0004])
    ops.mass(2588, *[0.0016, 0.0016, 0.0016])
    ops.mass(2589, *[0.0005, 0.0005, 0.0005])
    ops.mass(2590, *[0.0016, 0.0016, 0.0016])
    ops.mass(2591, *[0.0006, 0.0006, 0.0006])
    ops.mass(2592, *[0.0026, 0.0026, 0.0026])
    ops.mass(2593, *[0.0004, 0.0004, 0.0004])
    ops.mass(2594, *[0.002, 0.002, 0.002])
    ops.mass(2595, *[0.0006, 0.0006, 0.0006])
    ops.mass(2596, *[0.0048, 0.0048, 0.0048])
    ops.mass(2597, *[0.0012, 0.0012, 0.0012])
    ops.mass(2598, *[0.001, 0.001, 0.001])
    ops.mass(2599, *[0.0003, 0.0003, 0.0003])
    ops.mass(2600, *[0.0014, 0.0014, 0.0014])
    ops.mass(2601, *[0.0004, 0.0004, 0.0004])
    ops.mass(2602, *[0.0037, 0.0037, 0.0037])
    ops.mass(2603, *[0.0006, 0.0006, 0.0006])
    ops.mass(2604, *[0.0014, 0.0014, 0.0014])
    ops.mass(2605, *[0.0007, 0.0007, 0.0007])
    ops.mass(2606, *[0.0008, 0.0008, 0.0008])
    ops.mass(2607, *[0.0006, 0.0006, 0.0006])
    ops.mass(2608, *[0.0006, 0.0006, 0.0006])
    ops.mass(2609, *[0.0001, 0.0001, 0.0001])
    ops.mass(2610, *[0.0002, 0.0002, 0.0002])
    ops.mass(2611, *[0.0005, 0.0005, 0.0005])
    ops.mass(2612, *[0.0017, 0.0017, 0.0017])
    ops.mass(2613, *[0.0006, 0.0006, 0.0006])
    ops.mass(2614, *[0.0007, 0.0007, 0.0007])
    ops.mass(2615, *[0.0013, 0.0013, 0.0013])
    ops.mass(2616, *[0.0025, 0.0025, 0.0025])
    ops.mass(2617, *[0.0022, 0.0022, 0.0022])
    ops.mass(2618, *[0.0003, 0.0003, 0.0003])
    ops.mass(2619, *[0.0024, 0.0024, 0.0024])
    ops.mass(2620, *[0.0007, 0.0007, 0.0007])
    ops.mass(2621, *[0.0022, 0.0022, 0.0022])
    ops.mass(2622, *[0.0013, 0.0013, 0.0013])
    ops.mass(2623, *[0.0007, 0.0007, 0.0007])
    ops.mass(2624, *[0.0012, 0.0012, 0.0012])
    ops.mass(2625, *[0.0007, 0.0007, 0.0007])
    ops.mass(2626, *[0.001, 0.001, 0.001])
    ops.mass(2627, *[0.0009, 0.0009, 0.0009])
    ops.mass(2628, *[0.0003, 0.0003, 0.0003])
    ops.mass(2629, *[0.0003, 0.0003, 0.0003])
    ops.mass(2630, *[0.0013, 0.0013, 0.0013])
    ops.mass(2631, *[0.0022, 0.0022, 0.0022])
    ops.mass(2632, *[0.0011, 0.0011, 0.0011])
    ops.mass(2633, *[0.0008, 0.0008, 0.0008])
    ops.mass(2634, *[0.0017, 0.0017, 0.0017])
    ops.mass(2635, *[0.0016, 0.0016, 0.0016])
    ops.mass(2636, *[0.0003, 0.0003, 0.0003])
    ops.mass(2637, *[0.0008, 0.0008, 0.0008])
    ops.mass(2638, *[0.0011, 0.0011, 0.0011])
    ops.mass(2639, *[0.0006, 0.0006, 0.0006])
    ops.mass(2640, *[0.001, 0.001, 0.001])
    ops.mass(2641, *[0.0015, 0.0015, 0.0015])
    ops.mass(2642, *[0.0015, 0.0015, 0.0015])
    ops.mass(2643, *[0.0002, 0.0002, 0.0002])
    ops.mass(2644, *[0.0008, 0.0008, 0.0008])
    ops.mass(2645, *[0.0011, 0.0011, 0.0011])
    ops.mass(2646, *[0.0019, 0.0019, 0.0019])
    ops.mass(2647, *[0.0009, 0.0009, 0.0009])
    ops.mass(2648, *[0.0003, 0.0003, 0.0003])
    ops.mass(2649, *[0.0002, 0.0002, 0.0002])
    ops.mass(2650, *[0.0002, 0.0002, 0.0002])
    ops.mass(2651, *[0.0006, 0.0006, 0.0006])
    ops.mass(2652, *[0.0017, 0.0017, 0.0017])
    ops.mass(2653, *[0.0014, 0.0014, 0.0014])
    ops.mass(2654, *[0.0004, 0.0004, 0.0004])
    ops.mass(2655, *[0.0009, 0.0009, 0.0009])
    ops.mass(2656, *[0.0007, 0.0007, 0.0007])
    ops.mass(2657, *[0.0008, 0.0008, 0.0008])
    ops.mass(2658, *[0.0002, 0.0002, 0.0002])
    ops.mass(2659, *[0.0002, 0.0002, 0.0002])
    ops.mass(2660, *[0.0017, 0.0017, 0.0017])
    ops.mass(2661, *[0.0009, 0.0009, 0.0009])
    ops.mass(2662, *[0.0013, 0.0013, 0.0013])
    ops.mass(2663, *[0.0008, 0.0008, 0.0008])
    ops.mass(2664, *[0.0004, 0.0004, 0.0004])
    ops.mass(2665, *[0.0006, 0.0006, 0.0006])
    ops.mass(2666, *[0.0006, 0.0006, 0.0006])
    ops.mass(2667, *[0.0007, 0.0007, 0.0007])
    ops.mass(2668, *[0.0011, 0.0011, 0.0011])
    ops.mass(2669, *[0.0009, 0.0009, 0.0009])
    ops.mass(2670, *[0.0004, 0.0004, 0.0004])
    ops.mass(2671, *[0.0006, 0.0006, 0.0006])
    ops.mass(2672, *[0.0001, 0.0001, 0.0001])
    ops.mass(2673, *[0.0004, 0.0004, 0.0004])
    ops.mass(2674, *[0.0007, 0.0007, 0.0007])
    ops.mass(2675, *[0.0008, 0.0008, 0.0008])
    ops.mass(2676, *[0.0008, 0.0008, 0.0008])
    ops.mass(2677, *[0.0005, 0.0005, 0.0005])
    ops.mass(2678, *[0.0003, 0.0003, 0.0003])
    ops.mass(2679, *[0.0004, 0.0004, 0.0004])
    ops.mass(2680, *[0.0006, 0.0006, 0.0006])
    ops.mass(2681, *[0.0018, 0.0018, 0.0018])
    ops.mass(2682, *[0.0027, 0.0027, 0.0027])
    ops.mass(2683, *[0.0007, 0.0007, 0.0007])
    ops.mass(2684, *[0.0004, 0.0004, 0.0004])
    ops.mass(2685, *[0.0001, 0.0001, 0.0001])
    ops.mass(2686, *[0.0004, 0.0004, 0.0004])
    ops.mass(2687, *[0.0003, 0.0003, 0.0003])
    return ops

Igloo().export("model.msh") #vtk") #print("-json")
